<?php

/**
 * Base class that represents a row from the 'katao_cart' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoCart extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoCartPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_user_id field.
	 * @var        int
	 */
	protected $katao_user_id;


	/**
	 * The value for the katao_period_id field.
	 * @var        int
	 */
	protected $katao_period_id;


	/**
	 * The value for the katao_node_id field.
	 * @var        int
	 */
	protected $katao_node_id;


	/**
	 * The value for the status field.
	 * @var        int
	 */
	protected $status = 1;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoUser
	 */
	protected $aKataoUser;

	/**
	 * @var        KataoPeriod
	 */
	protected $aKataoPeriod;

	/**
	 * @var        KataoNode
	 */
	protected $aKataoNode;

	/**
	 * Collection to store aggregation of collKataoCartProducts.
	 * @var        array
	 */
	protected $collKataoCartProducts;

	/**
	 * The criteria used to select the current contents of collKataoCartProducts.
	 * @var        Criteria
	 */
	protected $lastKataoCartProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoInvoices.
	 * @var        array
	 */
	protected $collKataoInvoices;

	/**
	 * The criteria used to select the current contents of collKataoInvoices.
	 * @var        Criteria
	 */
	protected $lastKataoInvoiceCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_user_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoUserId()
	{

		return $this->katao_user_id;
	}

	/**
	 * Get the [katao_period_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoPeriodId()
	{

		return $this->katao_period_id;
	}

	/**
	 * Get the [katao_node_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoNodeId()
	{

		return $this->katao_node_id;
	}

	/**
	 * Get the [status] column value.
	 * 
	 * @return     int
	 */
	public function getStatus()
	{

		return $this->status;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoCartPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_user_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoUserId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_user_id !== $v) {
			$this->katao_user_id = $v;
			$this->modifiedColumns[] = KataoCartPeer::KATAO_USER_ID;
		}

		if ($this->aKataoUser !== null && $this->aKataoUser->getId() !== $v) {
			$this->aKataoUser = null;
		}

	} // setKataoUserId()

	/**
	 * Set the value of [katao_period_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoPeriodId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_period_id !== $v) {
			$this->katao_period_id = $v;
			$this->modifiedColumns[] = KataoCartPeer::KATAO_PERIOD_ID;
		}

		if ($this->aKataoPeriod !== null && $this->aKataoPeriod->getId() !== $v) {
			$this->aKataoPeriod = null;
		}

	} // setKataoPeriodId()

	/**
	 * Set the value of [katao_node_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoNodeId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_node_id !== $v) {
			$this->katao_node_id = $v;
			$this->modifiedColumns[] = KataoCartPeer::KATAO_NODE_ID;
		}

		if ($this->aKataoNode !== null && $this->aKataoNode->getId() !== $v) {
			$this->aKataoNode = null;
		}

	} // setKataoNodeId()

	/**
	 * Set the value of [status] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setStatus($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->status !== $v || $v === 1) {
			$this->status = $v;
			$this->modifiedColumns[] = KataoCartPeer::STATUS;
		}

	} // setStatus()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoCartPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoCartPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_user_id = $rs->getInt($startcol + 1);

			$this->katao_period_id = $rs->getInt($startcol + 2);

			$this->katao_node_id = $rs->getInt($startcol + 3);

			$this->status = $rs->getInt($startcol + 4);

			$this->created_at = $rs->getTimestamp($startcol + 5, null);

			$this->updated_at = $rs->getTimestamp($startcol + 6, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 7; // 7 = KataoCartPeer::NUM_COLUMNS - KataoCartPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoCart object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoCart:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoCartPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoCartPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoCart:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoCart:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoCartPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoCartPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoCartPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoCart:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoUser !== null) {
				if ($this->aKataoUser->isModified()) {
					$affectedRows += $this->aKataoUser->save($con);
				}
				$this->setKataoUser($this->aKataoUser);
			}

			if ($this->aKataoPeriod !== null) {
				if ($this->aKataoPeriod->isModified()) {
					$affectedRows += $this->aKataoPeriod->save($con);
				}
				$this->setKataoPeriod($this->aKataoPeriod);
			}

			if ($this->aKataoNode !== null) {
				if ($this->aKataoNode->isModified()) {
					$affectedRows += $this->aKataoNode->save($con);
				}
				$this->setKataoNode($this->aKataoNode);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoCartPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoCartPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoCartProducts !== null) {
				foreach($this->collKataoCartProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoInvoices !== null) {
				foreach($this->collKataoInvoices as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoUser !== null) {
				if (!$this->aKataoUser->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoUser->getValidationFailures());
				}
			}

			if ($this->aKataoPeriod !== null) {
				if (!$this->aKataoPeriod->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoPeriod->getValidationFailures());
				}
			}

			if ($this->aKataoNode !== null) {
				if (!$this->aKataoNode->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoNode->getValidationFailures());
				}
			}


			if (($retval = KataoCartPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoCartProducts !== null) {
					foreach($this->collKataoCartProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoInvoices !== null) {
					foreach($this->collKataoInvoices as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoCartPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoUserId();
				break;
			case 2:
				return $this->getKataoPeriodId();
				break;
			case 3:
				return $this->getKataoNodeId();
				break;
			case 4:
				return $this->getStatus();
				break;
			case 5:
				return $this->getCreatedAt();
				break;
			case 6:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoCartPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoUserId(),
			$keys[2] => $this->getKataoPeriodId(),
			$keys[3] => $this->getKataoNodeId(),
			$keys[4] => $this->getStatus(),
			$keys[5] => $this->getCreatedAt(),
			$keys[6] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoCartPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoUserId($value);
				break;
			case 2:
				$this->setKataoPeriodId($value);
				break;
			case 3:
				$this->setKataoNodeId($value);
				break;
			case 4:
				$this->setStatus($value);
				break;
			case 5:
				$this->setCreatedAt($value);
				break;
			case 6:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoCartPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoUserId($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setKataoPeriodId($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setKataoNodeId($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setStatus($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setCreatedAt($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setUpdatedAt($arr[$keys[6]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoCartPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoCartPeer::ID)) $criteria->add(KataoCartPeer::ID, $this->id);
		if ($this->isColumnModified(KataoCartPeer::KATAO_USER_ID)) $criteria->add(KataoCartPeer::KATAO_USER_ID, $this->katao_user_id);
		if ($this->isColumnModified(KataoCartPeer::KATAO_PERIOD_ID)) $criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->katao_period_id);
		if ($this->isColumnModified(KataoCartPeer::KATAO_NODE_ID)) $criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->katao_node_id);
		if ($this->isColumnModified(KataoCartPeer::STATUS)) $criteria->add(KataoCartPeer::STATUS, $this->status);
		if ($this->isColumnModified(KataoCartPeer::CREATED_AT)) $criteria->add(KataoCartPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoCartPeer::UPDATED_AT)) $criteria->add(KataoCartPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoCartPeer::DATABASE_NAME);

		$criteria->add(KataoCartPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoCart (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoUserId($this->katao_user_id);

		$copyObj->setKataoPeriodId($this->katao_period_id);

		$copyObj->setKataoNodeId($this->katao_node_id);

		$copyObj->setStatus($this->status);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoCartProducts() as $relObj) {
				$copyObj->addKataoCartProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoInvoices() as $relObj) {
				$copyObj->addKataoInvoice($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoCart Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoCartPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoCartPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoUser object.
	 *
	 * @param      KataoUser $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoUser($v)
	{


		if ($v === null) {
			$this->setKataoUserId(NULL);
		} else {
			$this->setKataoUserId($v->getId());
		}


		$this->aKataoUser = $v;
	}


	/**
	 * Get the associated KataoUser object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoUser The associated KataoUser object.
	 * @throws     PropelException
	 */
	public function getKataoUser($con = null)
	{
		if ($this->aKataoUser === null && ($this->katao_user_id !== null)) {
			// include the related Peer class
			$this->aKataoUser = KataoUserPeer::retrieveByPK($this->katao_user_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoUserPeer::retrieveByPK($this->katao_user_id, $con);
			   $obj->addKataoUsers($this);
			 */
		}
		return $this->aKataoUser;
	}

	/**
	 * Declares an association between this object and a KataoPeriod object.
	 *
	 * @param      KataoPeriod $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoPeriod($v)
	{


		if ($v === null) {
			$this->setKataoPeriodId(NULL);
		} else {
			$this->setKataoPeriodId($v->getId());
		}


		$this->aKataoPeriod = $v;
	}


	/**
	 * Get the associated KataoPeriod object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoPeriod The associated KataoPeriod object.
	 * @throws     PropelException
	 */
	public function getKataoPeriod($con = null)
	{
		if ($this->aKataoPeriod === null && ($this->katao_period_id !== null)) {
			// include the related Peer class
			$this->aKataoPeriod = KataoPeriodPeer::retrieveByPK($this->katao_period_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoPeriodPeer::retrieveByPK($this->katao_period_id, $con);
			   $obj->addKataoPeriods($this);
			 */
		}
		return $this->aKataoPeriod;
	}

	/**
	 * Declares an association between this object and a KataoNode object.
	 *
	 * @param      KataoNode $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoNode($v)
	{


		if ($v === null) {
			$this->setKataoNodeId(NULL);
		} else {
			$this->setKataoNodeId($v->getId());
		}


		$this->aKataoNode = $v;
	}


	/**
	 * Get the associated KataoNode object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoNode The associated KataoNode object.
	 * @throws     PropelException
	 */
	public function getKataoNode($con = null)
	{
		if ($this->aKataoNode === null && ($this->katao_node_id !== null)) {
			// include the related Peer class
			$this->aKataoNode = KataoNodePeer::retrieveByPK($this->katao_node_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoNodePeer::retrieveByPK($this->katao_node_id, $con);
			   $obj->addKataoNodes($this);
			 */
		}
		return $this->aKataoNode;
	}

	/**
	 * Temporary storage of collKataoCartProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoCartProducts()
	{
		if ($this->collKataoCartProducts === null) {
			$this->collKataoCartProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 * If this KataoCart is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoCartProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
					$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;
		return $this->collKataoCartProducts;
	}

	/**
	 * Returns the number of related KataoCartProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoCartProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

		return KataoCartProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoCartProduct object to this object
	 * through the KataoCartProduct foreign key attribute
	 *
	 * @param      KataoCartProduct $l KataoCartProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoCartProduct(KataoCartProduct $l)
	{
		$this->collKataoCartProducts[] = $l;
		$l->setKataoCart($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart is new, it will return
	 * an empty collection; or if this KataoCart has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoCart.
	 */
	public function getKataoCartProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart is new, it will return
	 * an empty collection; or if this KataoCart has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoCart.
	 */
	public function getKataoCartProductsJoinKataoOrderProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_CART_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}

	/**
	 * Temporary storage of collKataoInvoices to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoInvoices()
	{
		if ($this->collKataoInvoices === null) {
			$this->collKataoInvoices = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 * If this KataoCart is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoInvoices($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
			   $this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
					$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;
		return $this->collKataoInvoices;
	}

	/**
	 * Returns the number of related KataoInvoices.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoInvoices($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

		return KataoInvoicePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoInvoice object to this object
	 * through the KataoInvoice foreign key attribute
	 *
	 * @param      KataoInvoice $l KataoInvoice
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoInvoice(KataoInvoice $l)
	{
		$this->collKataoInvoices[] = $l;
		$l->setKataoCart($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart is new, it will return
	 * an empty collection; or if this KataoCart has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoCart.
	 */
	public function getKataoInvoicesJoinKataoMember($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart is new, it will return
	 * an empty collection; or if this KataoCart has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoCart.
	 */
	public function getKataoInvoicesJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoCart is new, it will return
	 * an empty collection; or if this KataoCart has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoCart.
	 */
	public function getKataoInvoicesJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_CART_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoCart:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoCart::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoCart
