<?php

/**
 * Base class that represents a row from the 'katao_member_deposit' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoMemberDeposit extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoMemberDepositPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_member_id field.
	 * @var        int
	 */
	protected $katao_member_id;


	/**
	 * The value for the amount field.
	 * @var        double
	 */
	protected $amount;


	/**
	 * The value for the comment field.
	 * @var        string
	 */
	protected $comment;


	/**
	 * The value for the currency field.
	 * @var        int
	 */
	protected $currency = 1;


	/**
	 * The value for the payment_mode field.
	 * @var        int
	 */
	protected $payment_mode = 0;


	/**
	 * The value for the check_number field.
	 * @var        string
	 */
	protected $check_number;


	/**
	 * The value for the check_bank field.
	 * @var        string
	 */
	protected $check_bank;


	/**
	 * The value for the check_deposit field.
	 * @var        string
	 */
	protected $check_deposit;


	/**
	 * The value for the status field.
	 * @var        int
	 */
	protected $status = 1;


	/**
	 * The value for the valued_at field.
	 * @var        int
	 */
	protected $valued_at;


	/**
	 * The value for the valid_until field.
	 * @var        int
	 */
	protected $valid_until;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoMember
	 */
	protected $aKataoMember;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_member_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoMemberId()
	{

		return $this->katao_member_id;
	}

	/**
	 * Get the [amount] column value.
	 * 
	 * @return     double
	 */
	public function getAmount()
	{

		return $this->amount;
	}

	/**
	 * Get the [comment] column value.
	 * 
	 * @return     string
	 */
	public function getComment()
	{

		return $this->comment;
	}

	/**
	 * Get the [currency] column value.
	 * 
	 * @return     int
	 */
	public function getCurrency()
	{

		return $this->currency;
	}

	/**
	 * Get the [payment_mode] column value.
	 * 
	 * @return     int
	 */
	public function getPaymentMode()
	{

		return $this->payment_mode;
	}

	/**
	 * Get the [check_number] column value.
	 * 
	 * @return     string
	 */
	public function getCheckNumber()
	{

		return $this->check_number;
	}

	/**
	 * Get the [check_bank] column value.
	 * 
	 * @return     string
	 */
	public function getCheckBank()
	{

		return $this->check_bank;
	}

	/**
	 * Get the [check_deposit] column value.
	 * 
	 * @return     string
	 */
	public function getCheckDeposit()
	{

		return $this->check_deposit;
	}

	/**
	 * Get the [status] column value.
	 * 
	 * @return     int
	 */
	public function getStatus()
	{

		return $this->status;
	}

	/**
	 * Get the [optionally formatted] [valued_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getValuedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->valued_at === null || $this->valued_at === '') {
			return null;
		} elseif (!is_int($this->valued_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->valued_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [valued_at] as date/time value: " . var_export($this->valued_at, true));
			}
		} else {
			$ts = $this->valued_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [valid_until] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getValidUntil($format = 'Y-m-d H:i:s')
	{

		if ($this->valid_until === null || $this->valid_until === '') {
			return null;
		} elseif (!is_int($this->valid_until)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->valid_until);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [valid_until] as date/time value: " . var_export($this->valid_until, true));
			}
		} else {
			$ts = $this->valid_until;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_member_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoMemberId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_member_id !== $v) {
			$this->katao_member_id = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::KATAO_MEMBER_ID;
		}

		if ($this->aKataoMember !== null && $this->aKataoMember->getId() !== $v) {
			$this->aKataoMember = null;
		}

	} // setKataoMemberId()

	/**
	 * Set the value of [amount] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setAmount($v)
	{

		if ($this->amount !== $v) {
			$this->amount = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::AMOUNT;
		}

	} // setAmount()

	/**
	 * Set the value of [comment] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setComment($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->comment !== $v) {
			$this->comment = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::COMMENT;
		}

	} // setComment()

	/**
	 * Set the value of [currency] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCurrency($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->currency !== $v || $v === 1) {
			$this->currency = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::CURRENCY;
		}

	} // setCurrency()

	/**
	 * Set the value of [payment_mode] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setPaymentMode($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->payment_mode !== $v || $v === 0) {
			$this->payment_mode = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::PAYMENT_MODE;
		}

	} // setPaymentMode()

	/**
	 * Set the value of [check_number] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setCheckNumber($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->check_number !== $v) {
			$this->check_number = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::CHECK_NUMBER;
		}

	} // setCheckNumber()

	/**
	 * Set the value of [check_bank] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setCheckBank($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->check_bank !== $v) {
			$this->check_bank = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::CHECK_BANK;
		}

	} // setCheckBank()

	/**
	 * Set the value of [check_deposit] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setCheckDeposit($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->check_deposit !== $v) {
			$this->check_deposit = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::CHECK_DEPOSIT;
		}

	} // setCheckDeposit()

	/**
	 * Set the value of [status] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setStatus($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->status !== $v || $v === 1) {
			$this->status = $v;
			$this->modifiedColumns[] = KataoMemberDepositPeer::STATUS;
		}

	} // setStatus()

	/**
	 * Set the value of [valued_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setValuedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [valued_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->valued_at !== $ts) {
			$this->valued_at = $ts;
			$this->modifiedColumns[] = KataoMemberDepositPeer::VALUED_AT;
		}

	} // setValuedAt()

	/**
	 * Set the value of [valid_until] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setValidUntil($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [valid_until] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->valid_until !== $ts) {
			$this->valid_until = $ts;
			$this->modifiedColumns[] = KataoMemberDepositPeer::VALID_UNTIL;
		}

	} // setValidUntil()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoMemberDepositPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoMemberDepositPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_member_id = $rs->getInt($startcol + 1);

			$this->amount = $rs->getFloat($startcol + 2);

			$this->comment = $rs->getString($startcol + 3);

			$this->currency = $rs->getInt($startcol + 4);

			$this->payment_mode = $rs->getInt($startcol + 5);

			$this->check_number = $rs->getString($startcol + 6);

			$this->check_bank = $rs->getString($startcol + 7);

			$this->check_deposit = $rs->getString($startcol + 8);

			$this->status = $rs->getInt($startcol + 9);

			$this->valued_at = $rs->getTimestamp($startcol + 10, null);

			$this->valid_until = $rs->getTimestamp($startcol + 11, null);

			$this->created_at = $rs->getTimestamp($startcol + 12, null);

			$this->updated_at = $rs->getTimestamp($startcol + 13, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 14; // 14 = KataoMemberDepositPeer::NUM_COLUMNS - KataoMemberDepositPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoMemberDeposit object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoMemberDeposit:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoMemberDepositPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoMemberDepositPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoMemberDeposit:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoMemberDeposit:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoMemberDepositPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoMemberDepositPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoMemberDepositPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoMemberDeposit:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoMember !== null) {
				if ($this->aKataoMember->isModified()) {
					$affectedRows += $this->aKataoMember->save($con);
				}
				$this->setKataoMember($this->aKataoMember);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoMemberDepositPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoMemberDepositPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoMember !== null) {
				if (!$this->aKataoMember->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoMember->getValidationFailures());
				}
			}


			if (($retval = KataoMemberDepositPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}



			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoMemberDepositPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoMemberId();
				break;
			case 2:
				return $this->getAmount();
				break;
			case 3:
				return $this->getComment();
				break;
			case 4:
				return $this->getCurrency();
				break;
			case 5:
				return $this->getPaymentMode();
				break;
			case 6:
				return $this->getCheckNumber();
				break;
			case 7:
				return $this->getCheckBank();
				break;
			case 8:
				return $this->getCheckDeposit();
				break;
			case 9:
				return $this->getStatus();
				break;
			case 10:
				return $this->getValuedAt();
				break;
			case 11:
				return $this->getValidUntil();
				break;
			case 12:
				return $this->getCreatedAt();
				break;
			case 13:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoMemberDepositPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoMemberId(),
			$keys[2] => $this->getAmount(),
			$keys[3] => $this->getComment(),
			$keys[4] => $this->getCurrency(),
			$keys[5] => $this->getPaymentMode(),
			$keys[6] => $this->getCheckNumber(),
			$keys[7] => $this->getCheckBank(),
			$keys[8] => $this->getCheckDeposit(),
			$keys[9] => $this->getStatus(),
			$keys[10] => $this->getValuedAt(),
			$keys[11] => $this->getValidUntil(),
			$keys[12] => $this->getCreatedAt(),
			$keys[13] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoMemberDepositPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoMemberId($value);
				break;
			case 2:
				$this->setAmount($value);
				break;
			case 3:
				$this->setComment($value);
				break;
			case 4:
				$this->setCurrency($value);
				break;
			case 5:
				$this->setPaymentMode($value);
				break;
			case 6:
				$this->setCheckNumber($value);
				break;
			case 7:
				$this->setCheckBank($value);
				break;
			case 8:
				$this->setCheckDeposit($value);
				break;
			case 9:
				$this->setStatus($value);
				break;
			case 10:
				$this->setValuedAt($value);
				break;
			case 11:
				$this->setValidUntil($value);
				break;
			case 12:
				$this->setCreatedAt($value);
				break;
			case 13:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoMemberDepositPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoMemberId($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setAmount($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setComment($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setCurrency($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setPaymentMode($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setCheckNumber($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setCheckBank($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setCheckDeposit($arr[$keys[8]]);
		if (array_key_exists($keys[9], $arr)) $this->setStatus($arr[$keys[9]]);
		if (array_key_exists($keys[10], $arr)) $this->setValuedAt($arr[$keys[10]]);
		if (array_key_exists($keys[11], $arr)) $this->setValidUntil($arr[$keys[11]]);
		if (array_key_exists($keys[12], $arr)) $this->setCreatedAt($arr[$keys[12]]);
		if (array_key_exists($keys[13], $arr)) $this->setUpdatedAt($arr[$keys[13]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoMemberDepositPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoMemberDepositPeer::ID)) $criteria->add(KataoMemberDepositPeer::ID, $this->id);
		if ($this->isColumnModified(KataoMemberDepositPeer::KATAO_MEMBER_ID)) $criteria->add(KataoMemberDepositPeer::KATAO_MEMBER_ID, $this->katao_member_id);
		if ($this->isColumnModified(KataoMemberDepositPeer::AMOUNT)) $criteria->add(KataoMemberDepositPeer::AMOUNT, $this->amount);
		if ($this->isColumnModified(KataoMemberDepositPeer::COMMENT)) $criteria->add(KataoMemberDepositPeer::COMMENT, $this->comment);
		if ($this->isColumnModified(KataoMemberDepositPeer::CURRENCY)) $criteria->add(KataoMemberDepositPeer::CURRENCY, $this->currency);
		if ($this->isColumnModified(KataoMemberDepositPeer::PAYMENT_MODE)) $criteria->add(KataoMemberDepositPeer::PAYMENT_MODE, $this->payment_mode);
		if ($this->isColumnModified(KataoMemberDepositPeer::CHECK_NUMBER)) $criteria->add(KataoMemberDepositPeer::CHECK_NUMBER, $this->check_number);
		if ($this->isColumnModified(KataoMemberDepositPeer::CHECK_BANK)) $criteria->add(KataoMemberDepositPeer::CHECK_BANK, $this->check_bank);
		if ($this->isColumnModified(KataoMemberDepositPeer::CHECK_DEPOSIT)) $criteria->add(KataoMemberDepositPeer::CHECK_DEPOSIT, $this->check_deposit);
		if ($this->isColumnModified(KataoMemberDepositPeer::STATUS)) $criteria->add(KataoMemberDepositPeer::STATUS, $this->status);
		if ($this->isColumnModified(KataoMemberDepositPeer::VALUED_AT)) $criteria->add(KataoMemberDepositPeer::VALUED_AT, $this->valued_at);
		if ($this->isColumnModified(KataoMemberDepositPeer::VALID_UNTIL)) $criteria->add(KataoMemberDepositPeer::VALID_UNTIL, $this->valid_until);
		if ($this->isColumnModified(KataoMemberDepositPeer::CREATED_AT)) $criteria->add(KataoMemberDepositPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoMemberDepositPeer::UPDATED_AT)) $criteria->add(KataoMemberDepositPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoMemberDepositPeer::DATABASE_NAME);

		$criteria->add(KataoMemberDepositPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoMemberDeposit (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoMemberId($this->katao_member_id);

		$copyObj->setAmount($this->amount);

		$copyObj->setComment($this->comment);

		$copyObj->setCurrency($this->currency);

		$copyObj->setPaymentMode($this->payment_mode);

		$copyObj->setCheckNumber($this->check_number);

		$copyObj->setCheckBank($this->check_bank);

		$copyObj->setCheckDeposit($this->check_deposit);

		$copyObj->setStatus($this->status);

		$copyObj->setValuedAt($this->valued_at);

		$copyObj->setValidUntil($this->valid_until);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoMemberDeposit Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoMemberDepositPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoMemberDepositPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoMember object.
	 *
	 * @param      KataoMember $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoMember($v)
	{


		if ($v === null) {
			$this->setKataoMemberId(NULL);
		} else {
			$this->setKataoMemberId($v->getId());
		}


		$this->aKataoMember = $v;
	}


	/**
	 * Get the associated KataoMember object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoMember The associated KataoMember object.
	 * @throws     PropelException
	 */
	public function getKataoMember($con = null)
	{
		if ($this->aKataoMember === null && ($this->katao_member_id !== null)) {
			// include the related Peer class
			$this->aKataoMember = KataoMemberPeer::retrieveByPK($this->katao_member_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoMemberPeer::retrieveByPK($this->katao_member_id, $con);
			   $obj->addKataoMembers($this);
			 */
		}
		return $this->aKataoMember;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoMemberDeposit:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoMemberDeposit::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoMemberDeposit
