<?php

/**
 * Base class that represents a row from the 'katao_node' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoNode extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoNodePeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the city field.
	 * @var        string
	 */
	protected $city;


	/**
	 * The value for the responsible field.
	 * @var        int
	 */
	protected $responsible;


	/**
	 * The value for the begin_at field.
	 * @var        int
	 */
	protected $begin_at;


	/**
	 * The value for the is_default field.
	 * @var        int
	 */
	protected $is_default = 0;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoUser
	 */
	protected $aKataoUser;

	/**
	 * Collection to store aggregation of collKataoMembersRelatedByKataoNodeId.
	 * @var        array
	 */
	protected $collKataoMembersRelatedByKataoNodeId;

	/**
	 * The criteria used to select the current contents of collKataoMembersRelatedByKataoNodeId.
	 * @var        Criteria
	 */
	protected $lastKataoMemberRelatedByKataoNodeIdCriteria = null;

	/**
	 * Collection to store aggregation of collKataoMembersRelatedByNextKataoNodeId.
	 * @var        array
	 */
	protected $collKataoMembersRelatedByNextKataoNodeId;

	/**
	 * The criteria used to select the current contents of collKataoMembersRelatedByNextKataoNodeId.
	 * @var        Criteria
	 */
	protected $lastKataoMemberRelatedByNextKataoNodeIdCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierNodes.
	 * @var        array
	 */
	protected $collKataoSupplierNodes;

	/**
	 * The criteria used to select the current contents of collKataoSupplierNodes.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierNodeCriteria = null;

	/**
	 * Collection to store aggregation of collKataoNodeProducts.
	 * @var        array
	 */
	protected $collKataoNodeProducts;

	/**
	 * The criteria used to select the current contents of collKataoNodeProducts.
	 * @var        Criteria
	 */
	protected $lastKataoNodeProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoNodeSuppliers.
	 * @var        array
	 */
	protected $collKataoNodeSuppliers;

	/**
	 * The criteria used to select the current contents of collKataoNodeSuppliers.
	 * @var        Criteria
	 */
	protected $lastKataoNodeSupplierCriteria = null;

	/**
	 * Collection to store aggregation of collKataoCarts.
	 * @var        array
	 */
	protected $collKataoCarts;

	/**
	 * The criteria used to select the current contents of collKataoCarts.
	 * @var        Criteria
	 */
	protected $lastKataoCartCriteria = null;

	/**
	 * Collection to store aggregation of collKataoOrders.
	 * @var        array
	 */
	protected $collKataoOrders;

	/**
	 * The criteria used to select the current contents of collKataoOrders.
	 * @var        Criteria
	 */
	protected $lastKataoOrderCriteria = null;

	/**
	 * Collection to store aggregation of collKataoInvoices.
	 * @var        array
	 */
	protected $collKataoInvoices;

	/**
	 * The criteria used to select the current contents of collKataoInvoices.
	 * @var        Criteria
	 */
	protected $lastKataoInvoiceCriteria = null;

	/**
	 * Collection to store aggregation of collKataoMemberNodes.
	 * @var        array
	 */
	protected $collKataoMemberNodes;

	/**
	 * The criteria used to select the current contents of collKataoMemberNodes.
	 * @var        Criteria
	 */
	protected $lastKataoMemberNodeCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [city] column value.
	 * 
	 * @return     string
	 */
	public function getCity()
	{

		return $this->city;
	}

	/**
	 * Get the [responsible] column value.
	 * 
	 * @return     int
	 */
	public function getResponsible()
	{

		return $this->responsible;
	}

	/**
	 * Get the [optionally formatted] [begin_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getBeginAt($format = 'Y-m-d H:i:s')
	{

		if ($this->begin_at === null || $this->begin_at === '') {
			return null;
		} elseif (!is_int($this->begin_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->begin_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [begin_at] as date/time value: " . var_export($this->begin_at, true));
			}
		} else {
			$ts = $this->begin_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [is_default] column value.
	 * 
	 * @return     int
	 */
	public function getIsDefault()
	{

		return $this->is_default;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoNodePeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [city] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setCity($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->city !== $v) {
			$this->city = $v;
			$this->modifiedColumns[] = KataoNodePeer::CITY;
		}

	} // setCity()

	/**
	 * Set the value of [responsible] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setResponsible($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->responsible !== $v) {
			$this->responsible = $v;
			$this->modifiedColumns[] = KataoNodePeer::RESPONSIBLE;
		}

		if ($this->aKataoUser !== null && $this->aKataoUser->getId() !== $v) {
			$this->aKataoUser = null;
		}

	} // setResponsible()

	/**
	 * Set the value of [begin_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setBeginAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [begin_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->begin_at !== $ts) {
			$this->begin_at = $ts;
			$this->modifiedColumns[] = KataoNodePeer::BEGIN_AT;
		}

	} // setBeginAt()

	/**
	 * Set the value of [is_default] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setIsDefault($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->is_default !== $v || $v === 0) {
			$this->is_default = $v;
			$this->modifiedColumns[] = KataoNodePeer::IS_DEFAULT;
		}

	} // setIsDefault()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoNodePeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoNodePeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->city = $rs->getString($startcol + 1);

			$this->responsible = $rs->getInt($startcol + 2);

			$this->begin_at = $rs->getTimestamp($startcol + 3, null);

			$this->is_default = $rs->getInt($startcol + 4);

			$this->created_at = $rs->getTimestamp($startcol + 5, null);

			$this->updated_at = $rs->getTimestamp($startcol + 6, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 7; // 7 = KataoNodePeer::NUM_COLUMNS - KataoNodePeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoNode object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoNode:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoNodePeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoNodePeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoNode:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoNode:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoNodePeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoNodePeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoNodePeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoNode:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoUser !== null) {
				if ($this->aKataoUser->isModified()) {
					$affectedRows += $this->aKataoUser->save($con);
				}
				$this->setKataoUser($this->aKataoUser);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoNodePeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoNodePeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoMembersRelatedByKataoNodeId !== null) {
				foreach($this->collKataoMembersRelatedByKataoNodeId as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoMembersRelatedByNextKataoNodeId !== null) {
				foreach($this->collKataoMembersRelatedByNextKataoNodeId as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierNodes !== null) {
				foreach($this->collKataoSupplierNodes as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoNodeProducts !== null) {
				foreach($this->collKataoNodeProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoNodeSuppliers !== null) {
				foreach($this->collKataoNodeSuppliers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoCarts !== null) {
				foreach($this->collKataoCarts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoOrders !== null) {
				foreach($this->collKataoOrders as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoInvoices !== null) {
				foreach($this->collKataoInvoices as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoMemberNodes !== null) {
				foreach($this->collKataoMemberNodes as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoUser !== null) {
				if (!$this->aKataoUser->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoUser->getValidationFailures());
				}
			}


			if (($retval = KataoNodePeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoMembersRelatedByKataoNodeId !== null) {
					foreach($this->collKataoMembersRelatedByKataoNodeId as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoMembersRelatedByNextKataoNodeId !== null) {
					foreach($this->collKataoMembersRelatedByNextKataoNodeId as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierNodes !== null) {
					foreach($this->collKataoSupplierNodes as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoNodeProducts !== null) {
					foreach($this->collKataoNodeProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoNodeSuppliers !== null) {
					foreach($this->collKataoNodeSuppliers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoCarts !== null) {
					foreach($this->collKataoCarts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoOrders !== null) {
					foreach($this->collKataoOrders as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoInvoices !== null) {
					foreach($this->collKataoInvoices as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoMemberNodes !== null) {
					foreach($this->collKataoMemberNodes as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoNodePeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getCity();
				break;
			case 2:
				return $this->getResponsible();
				break;
			case 3:
				return $this->getBeginAt();
				break;
			case 4:
				return $this->getIsDefault();
				break;
			case 5:
				return $this->getCreatedAt();
				break;
			case 6:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoNodePeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getCity(),
			$keys[2] => $this->getResponsible(),
			$keys[3] => $this->getBeginAt(),
			$keys[4] => $this->getIsDefault(),
			$keys[5] => $this->getCreatedAt(),
			$keys[6] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoNodePeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setCity($value);
				break;
			case 2:
				$this->setResponsible($value);
				break;
			case 3:
				$this->setBeginAt($value);
				break;
			case 4:
				$this->setIsDefault($value);
				break;
			case 5:
				$this->setCreatedAt($value);
				break;
			case 6:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoNodePeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setCity($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setResponsible($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setBeginAt($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setIsDefault($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setCreatedAt($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setUpdatedAt($arr[$keys[6]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoNodePeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoNodePeer::ID)) $criteria->add(KataoNodePeer::ID, $this->id);
		if ($this->isColumnModified(KataoNodePeer::CITY)) $criteria->add(KataoNodePeer::CITY, $this->city);
		if ($this->isColumnModified(KataoNodePeer::RESPONSIBLE)) $criteria->add(KataoNodePeer::RESPONSIBLE, $this->responsible);
		if ($this->isColumnModified(KataoNodePeer::BEGIN_AT)) $criteria->add(KataoNodePeer::BEGIN_AT, $this->begin_at);
		if ($this->isColumnModified(KataoNodePeer::IS_DEFAULT)) $criteria->add(KataoNodePeer::IS_DEFAULT, $this->is_default);
		if ($this->isColumnModified(KataoNodePeer::CREATED_AT)) $criteria->add(KataoNodePeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoNodePeer::UPDATED_AT)) $criteria->add(KataoNodePeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoNodePeer::DATABASE_NAME);

		$criteria->add(KataoNodePeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoNode (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setCity($this->city);

		$copyObj->setResponsible($this->responsible);

		$copyObj->setBeginAt($this->begin_at);

		$copyObj->setIsDefault($this->is_default);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoMembersRelatedByKataoNodeId() as $relObj) {
				$copyObj->addKataoMemberRelatedByKataoNodeId($relObj->copy($deepCopy));
			}

			foreach($this->getKataoMembersRelatedByNextKataoNodeId() as $relObj) {
				$copyObj->addKataoMemberRelatedByNextKataoNodeId($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierNodes() as $relObj) {
				$copyObj->addKataoSupplierNode($relObj->copy($deepCopy));
			}

			foreach($this->getKataoNodeProducts() as $relObj) {
				$copyObj->addKataoNodeProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoNodeSuppliers() as $relObj) {
				$copyObj->addKataoNodeSupplier($relObj->copy($deepCopy));
			}

			foreach($this->getKataoCarts() as $relObj) {
				$copyObj->addKataoCart($relObj->copy($deepCopy));
			}

			foreach($this->getKataoOrders() as $relObj) {
				$copyObj->addKataoOrder($relObj->copy($deepCopy));
			}

			foreach($this->getKataoInvoices() as $relObj) {
				$copyObj->addKataoInvoice($relObj->copy($deepCopy));
			}

			foreach($this->getKataoMemberNodes() as $relObj) {
				$copyObj->addKataoMemberNode($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoNode Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoNodePeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoNodePeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoUser object.
	 *
	 * @param      KataoUser $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoUser($v)
	{


		if ($v === null) {
			$this->setResponsible(NULL);
		} else {
			$this->setResponsible($v->getId());
		}


		$this->aKataoUser = $v;
	}


	/**
	 * Get the associated KataoUser object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoUser The associated KataoUser object.
	 * @throws     PropelException
	 */
	public function getKataoUser($con = null)
	{
		if ($this->aKataoUser === null && ($this->responsible !== null)) {
			// include the related Peer class
			$this->aKataoUser = KataoUserPeer::retrieveByPK($this->responsible, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoUserPeer::retrieveByPK($this->responsible, $con);
			   $obj->addKataoUsers($this);
			 */
		}
		return $this->aKataoUser;
	}

	/**
	 * Temporary storage of collKataoMembersRelatedByKataoNodeId to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoMembersRelatedByKataoNodeId()
	{
		if ($this->collKataoMembersRelatedByKataoNodeId === null) {
			$this->collKataoMembersRelatedByKataoNodeId = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoMembersRelatedByKataoNodeId from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoMembersRelatedByKataoNodeId($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMembersRelatedByKataoNodeId === null) {
			if ($this->isNew()) {
			   $this->collKataoMembersRelatedByKataoNodeId = array();
			} else {

				$criteria->add(KataoMemberPeer::KATAO_NODE_ID, $this->getId());

				KataoMemberPeer::addSelectColumns($criteria);
				$this->collKataoMembersRelatedByKataoNodeId = KataoMemberPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoMemberPeer::KATAO_NODE_ID, $this->getId());

				KataoMemberPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoMemberRelatedByKataoNodeIdCriteria) || !$this->lastKataoMemberRelatedByKataoNodeIdCriteria->equals($criteria)) {
					$this->collKataoMembersRelatedByKataoNodeId = KataoMemberPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoMemberRelatedByKataoNodeIdCriteria = $criteria;
		return $this->collKataoMembersRelatedByKataoNodeId;
	}

	/**
	 * Returns the number of related KataoMembersRelatedByKataoNodeId.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoMembersRelatedByKataoNodeId($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoMemberPeer::KATAO_NODE_ID, $this->getId());

		return KataoMemberPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoMember object to this object
	 * through the KataoMember foreign key attribute
	 *
	 * @param      KataoMember $l KataoMember
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoMemberRelatedByKataoNodeId(KataoMember $l)
	{
		$this->collKataoMembersRelatedByKataoNodeId[] = $l;
		$l->setKataoNodeRelatedByKataoNodeId($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoMembersRelatedByKataoNodeId from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoMembersRelatedByKataoNodeIdJoinKataoMemberRelatedByKataoMemberId($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMembersRelatedByKataoNodeId === null) {
			if ($this->isNew()) {
				$this->collKataoMembersRelatedByKataoNodeId = array();
			} else {

				$criteria->add(KataoMemberPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoMembersRelatedByKataoNodeId = KataoMemberPeer::doSelectJoinKataoMemberRelatedByKataoMemberId($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoMemberPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoMemberRelatedByKataoNodeIdCriteria) || !$this->lastKataoMemberRelatedByKataoNodeIdCriteria->equals($criteria)) {
				$this->collKataoMembersRelatedByKataoNodeId = KataoMemberPeer::doSelectJoinKataoMemberRelatedByKataoMemberId($criteria, $con);
			}
		}
		$this->lastKataoMemberRelatedByKataoNodeIdCriteria = $criteria;

		return $this->collKataoMembersRelatedByKataoNodeId;
	}

	/**
	 * Temporary storage of collKataoMembersRelatedByNextKataoNodeId to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoMembersRelatedByNextKataoNodeId()
	{
		if ($this->collKataoMembersRelatedByNextKataoNodeId === null) {
			$this->collKataoMembersRelatedByNextKataoNodeId = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoMembersRelatedByNextKataoNodeId from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoMembersRelatedByNextKataoNodeId($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMembersRelatedByNextKataoNodeId === null) {
			if ($this->isNew()) {
			   $this->collKataoMembersRelatedByNextKataoNodeId = array();
			} else {

				$criteria->add(KataoMemberPeer::NEXT_KATAO_NODE_ID, $this->getId());

				KataoMemberPeer::addSelectColumns($criteria);
				$this->collKataoMembersRelatedByNextKataoNodeId = KataoMemberPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoMemberPeer::NEXT_KATAO_NODE_ID, $this->getId());

				KataoMemberPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoMemberRelatedByNextKataoNodeIdCriteria) || !$this->lastKataoMemberRelatedByNextKataoNodeIdCriteria->equals($criteria)) {
					$this->collKataoMembersRelatedByNextKataoNodeId = KataoMemberPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoMemberRelatedByNextKataoNodeIdCriteria = $criteria;
		return $this->collKataoMembersRelatedByNextKataoNodeId;
	}

	/**
	 * Returns the number of related KataoMembersRelatedByNextKataoNodeId.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoMembersRelatedByNextKataoNodeId($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoMemberPeer::NEXT_KATAO_NODE_ID, $this->getId());

		return KataoMemberPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoMember object to this object
	 * through the KataoMember foreign key attribute
	 *
	 * @param      KataoMember $l KataoMember
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoMemberRelatedByNextKataoNodeId(KataoMember $l)
	{
		$this->collKataoMembersRelatedByNextKataoNodeId[] = $l;
		$l->setKataoNodeRelatedByNextKataoNodeId($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoMembersRelatedByNextKataoNodeId from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoMembersRelatedByNextKataoNodeIdJoinKataoMemberRelatedByKataoMemberId($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMembersRelatedByNextKataoNodeId === null) {
			if ($this->isNew()) {
				$this->collKataoMembersRelatedByNextKataoNodeId = array();
			} else {

				$criteria->add(KataoMemberPeer::NEXT_KATAO_NODE_ID, $this->getId());

				$this->collKataoMembersRelatedByNextKataoNodeId = KataoMemberPeer::doSelectJoinKataoMemberRelatedByKataoMemberId($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoMemberPeer::NEXT_KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoMemberRelatedByNextKataoNodeIdCriteria) || !$this->lastKataoMemberRelatedByNextKataoNodeIdCriteria->equals($criteria)) {
				$this->collKataoMembersRelatedByNextKataoNodeId = KataoMemberPeer::doSelectJoinKataoMemberRelatedByKataoMemberId($criteria, $con);
			}
		}
		$this->lastKataoMemberRelatedByNextKataoNodeIdCriteria = $criteria;

		return $this->collKataoMembersRelatedByNextKataoNodeId;
	}

	/**
	 * Temporary storage of collKataoSupplierNodes to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierNodes()
	{
		if ($this->collKataoSupplierNodes === null) {
			$this->collKataoSupplierNodes = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoSupplierNodes from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierNodes($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierNodes === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierNodes = array();
			} else {

				$criteria->add(KataoSupplierNodePeer::KATAO_NODE_ID, $this->getId());

				KataoSupplierNodePeer::addSelectColumns($criteria);
				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierNodePeer::KATAO_NODE_ID, $this->getId());

				KataoSupplierNodePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierNodeCriteria) || !$this->lastKataoSupplierNodeCriteria->equals($criteria)) {
					$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierNodeCriteria = $criteria;
		return $this->collKataoSupplierNodes;
	}

	/**
	 * Returns the number of related KataoSupplierNodes.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierNodes($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierNodePeer::KATAO_NODE_ID, $this->getId());

		return KataoSupplierNodePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierNode object to this object
	 * through the KataoSupplierNode foreign key attribute
	 *
	 * @param      KataoSupplierNode $l KataoSupplierNode
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierNode(KataoSupplierNode $l)
	{
		$this->collKataoSupplierNodes[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoSupplierNodes from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoSupplierNodesJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierNodes === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierNodes = array();
			} else {

				$criteria->add(KataoSupplierNodePeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierNodePeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoSupplierNodeCriteria) || !$this->lastKataoSupplierNodeCriteria->equals($criteria)) {
				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoSupplierNodeCriteria = $criteria;

		return $this->collKataoSupplierNodes;
	}

	/**
	 * Temporary storage of collKataoNodeProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeProducts()
	{
		if ($this->collKataoNodeProducts === null) {
			$this->collKataoNodeProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
					$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;
		return $this->collKataoNodeProducts;
	}

	/**
	 * Returns the number of related KataoNodeProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

		return KataoNodeProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeProduct object to this object
	 * through the KataoNodeProduct foreign key attribute
	 *
	 * @param      KataoNodeProduct $l KataoNodeProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeProduct(KataoNodeProduct $l)
	{
		$this->collKataoNodeProducts[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoNodeProductsJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoNodeProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}

	/**
	 * Temporary storage of collKataoNodeSuppliers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeSuppliers()
	{
		if ($this->collKataoNodeSuppliers === null) {
			$this->collKataoNodeSuppliers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeSuppliers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
					$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;
		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Returns the number of related KataoNodeSuppliers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeSuppliers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

		return KataoNodeSupplierPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeSupplier object to this object
	 * through the KataoNodeSupplier foreign key attribute
	 *
	 * @param      KataoNodeSupplier $l KataoNodeSupplier
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeSupplier(KataoNodeSupplier $l)
	{
		$this->collKataoNodeSuppliers[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoNodeSuppliersJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoNodeSuppliersJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Temporary storage of collKataoCarts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoCarts()
	{
		if ($this->collKataoCarts === null) {
			$this->collKataoCarts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoCarts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
			   $this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

				KataoCartPeer::addSelectColumns($criteria);
				$this->collKataoCarts = KataoCartPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

				KataoCartPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
					$this->collKataoCarts = KataoCartPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoCartCriteria = $criteria;
		return $this->collKataoCarts;
	}

	/**
	 * Returns the number of related KataoCarts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoCarts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

		return KataoCartPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoCart object to this object
	 * through the KataoCart foreign key attribute
	 *
	 * @param      KataoCart $l KataoCart
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoCart(KataoCart $l)
	{
		$this->collKataoCarts[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoCartsJoinKataoUser($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
				$this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoUser($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoUser($criteria, $con);
			}
		}
		$this->lastKataoCartCriteria = $criteria;

		return $this->collKataoCarts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoCartsJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
				$this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoCartCriteria = $criteria;

		return $this->collKataoCarts;
	}

	/**
	 * Temporary storage of collKataoOrders to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoOrders()
	{
		if ($this->collKataoOrders === null) {
			$this->collKataoOrders = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoOrders($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
			   $this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
					$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoOrderCriteria = $criteria;
		return $this->collKataoOrders;
	}

	/**
	 * Returns the number of related KataoOrders.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoOrders($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

		return KataoOrderPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoOrder object to this object
	 * through the KataoOrder foreign key attribute
	 *
	 * @param      KataoOrder $l KataoOrder
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoOrder(KataoOrder $l)
	{
		$this->collKataoOrders[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoOrdersJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoOrdersJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}

	/**
	 * Temporary storage of collKataoInvoices to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoInvoices()
	{
		if ($this->collKataoInvoices === null) {
			$this->collKataoInvoices = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoInvoices($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
			   $this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
					$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;
		return $this->collKataoInvoices;
	}

	/**
	 * Returns the number of related KataoInvoices.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoInvoices($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

		return KataoInvoicePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoInvoice object to this object
	 * through the KataoInvoice foreign key attribute
	 *
	 * @param      KataoInvoice $l KataoInvoice
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoInvoice(KataoInvoice $l)
	{
		$this->collKataoInvoices[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoInvoicesJoinKataoMember($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoInvoicesJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoInvoicesJoinKataoCart($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoCart($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoCart($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}

	/**
	 * Temporary storage of collKataoMemberNodes to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoMemberNodes()
	{
		if ($this->collKataoMemberNodes === null) {
			$this->collKataoMemberNodes = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode has previously
	 * been saved, it will retrieve related KataoMemberNodes from storage.
	 * If this KataoNode is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoMemberNodes($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMemberNodes === null) {
			if ($this->isNew()) {
			   $this->collKataoMemberNodes = array();
			} else {

				$criteria->add(KataoMemberNodePeer::KATAO_NODE_ID, $this->getId());

				KataoMemberNodePeer::addSelectColumns($criteria);
				$this->collKataoMemberNodes = KataoMemberNodePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoMemberNodePeer::KATAO_NODE_ID, $this->getId());

				KataoMemberNodePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoMemberNodeCriteria) || !$this->lastKataoMemberNodeCriteria->equals($criteria)) {
					$this->collKataoMemberNodes = KataoMemberNodePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoMemberNodeCriteria = $criteria;
		return $this->collKataoMemberNodes;
	}

	/**
	 * Returns the number of related KataoMemberNodes.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoMemberNodes($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoMemberNodePeer::KATAO_NODE_ID, $this->getId());

		return KataoMemberNodePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoMemberNode object to this object
	 * through the KataoMemberNode foreign key attribute
	 *
	 * @param      KataoMemberNode $l KataoMemberNode
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoMemberNode(KataoMemberNode $l)
	{
		$this->collKataoMemberNodes[] = $l;
		$l->setKataoNode($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoNode is new, it will return
	 * an empty collection; or if this KataoNode has previously
	 * been saved, it will retrieve related KataoMemberNodes from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoNode.
	 */
	public function getKataoMemberNodesJoinKataoMember($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoMemberNodes === null) {
			if ($this->isNew()) {
				$this->collKataoMemberNodes = array();
			} else {

				$criteria->add(KataoMemberNodePeer::KATAO_NODE_ID, $this->getId());

				$this->collKataoMemberNodes = KataoMemberNodePeer::doSelectJoinKataoMember($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoMemberNodePeer::KATAO_NODE_ID, $this->getId());

			if (!isset($this->lastKataoMemberNodeCriteria) || !$this->lastKataoMemberNodeCriteria->equals($criteria)) {
				$this->collKataoMemberNodes = KataoMemberNodePeer::doSelectJoinKataoMember($criteria, $con);
			}
		}
		$this->lastKataoMemberNodeCriteria = $criteria;

		return $this->collKataoMemberNodes;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoNode:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoNode::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoNode
