<?php

/**
 * Base class that represents a row from the 'katao_node_product' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoNodeProduct extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoNodeProductPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_node_id field.
	 * @var        int
	 */
	protected $katao_node_id;


	/**
	 * The value for the katao_period_id field.
	 * @var        int
	 */
	protected $katao_period_id;


	/**
	 * The value for the katao_product_id field.
	 * @var        int
	 */
	protected $katao_product_id;

	/**
	 * @var        KataoNode
	 */
	protected $aKataoNode;

	/**
	 * @var        KataoPeriod
	 */
	protected $aKataoPeriod;

	/**
	 * @var        KataoProduct
	 */
	protected $aKataoProduct;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_node_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoNodeId()
	{

		return $this->katao_node_id;
	}

	/**
	 * Get the [katao_period_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoPeriodId()
	{

		return $this->katao_period_id;
	}

	/**
	 * Get the [katao_product_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoProductId()
	{

		return $this->katao_product_id;
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoNodeProductPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_node_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoNodeId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_node_id !== $v) {
			$this->katao_node_id = $v;
			$this->modifiedColumns[] = KataoNodeProductPeer::KATAO_NODE_ID;
		}

		if ($this->aKataoNode !== null && $this->aKataoNode->getId() !== $v) {
			$this->aKataoNode = null;
		}

	} // setKataoNodeId()

	/**
	 * Set the value of [katao_period_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoPeriodId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_period_id !== $v) {
			$this->katao_period_id = $v;
			$this->modifiedColumns[] = KataoNodeProductPeer::KATAO_PERIOD_ID;
		}

		if ($this->aKataoPeriod !== null && $this->aKataoPeriod->getId() !== $v) {
			$this->aKataoPeriod = null;
		}

	} // setKataoPeriodId()

	/**
	 * Set the value of [katao_product_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoProductId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_product_id !== $v) {
			$this->katao_product_id = $v;
			$this->modifiedColumns[] = KataoNodeProductPeer::KATAO_PRODUCT_ID;
		}

		if ($this->aKataoProduct !== null && $this->aKataoProduct->getId() !== $v) {
			$this->aKataoProduct = null;
		}

	} // setKataoProductId()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_node_id = $rs->getInt($startcol + 1);

			$this->katao_period_id = $rs->getInt($startcol + 2);

			$this->katao_product_id = $rs->getInt($startcol + 3);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 4; // 4 = KataoNodeProductPeer::NUM_COLUMNS - KataoNodeProductPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoNodeProduct object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoNodeProduct:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoNodeProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoNodeProductPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoNodeProduct:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoNodeProduct:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoNodeProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoNodeProduct:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoNode !== null) {
				if ($this->aKataoNode->isModified()) {
					$affectedRows += $this->aKataoNode->save($con);
				}
				$this->setKataoNode($this->aKataoNode);
			}

			if ($this->aKataoPeriod !== null) {
				if ($this->aKataoPeriod->isModified()) {
					$affectedRows += $this->aKataoPeriod->save($con);
				}
				$this->setKataoPeriod($this->aKataoPeriod);
			}

			if ($this->aKataoProduct !== null) {
				if ($this->aKataoProduct->isModified()) {
					$affectedRows += $this->aKataoProduct->save($con);
				}
				$this->setKataoProduct($this->aKataoProduct);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoNodeProductPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoNodeProductPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoNode !== null) {
				if (!$this->aKataoNode->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoNode->getValidationFailures());
				}
			}

			if ($this->aKataoPeriod !== null) {
				if (!$this->aKataoPeriod->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoPeriod->getValidationFailures());
				}
			}

			if ($this->aKataoProduct !== null) {
				if (!$this->aKataoProduct->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoProduct->getValidationFailures());
				}
			}


			if (($retval = KataoNodeProductPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}



			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoNodeProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoNodeId();
				break;
			case 2:
				return $this->getKataoPeriodId();
				break;
			case 3:
				return $this->getKataoProductId();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoNodeProductPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoNodeId(),
			$keys[2] => $this->getKataoPeriodId(),
			$keys[3] => $this->getKataoProductId(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoNodeProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoNodeId($value);
				break;
			case 2:
				$this->setKataoPeriodId($value);
				break;
			case 3:
				$this->setKataoProductId($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoNodeProductPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoNodeId($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setKataoPeriodId($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setKataoProductId($arr[$keys[3]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoNodeProductPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoNodeProductPeer::ID)) $criteria->add(KataoNodeProductPeer::ID, $this->id);
		if ($this->isColumnModified(KataoNodeProductPeer::KATAO_NODE_ID)) $criteria->add(KataoNodeProductPeer::KATAO_NODE_ID, $this->katao_node_id);
		if ($this->isColumnModified(KataoNodeProductPeer::KATAO_PERIOD_ID)) $criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->katao_period_id);
		if ($this->isColumnModified(KataoNodeProductPeer::KATAO_PRODUCT_ID)) $criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->katao_product_id);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoNodeProductPeer::DATABASE_NAME);

		$criteria->add(KataoNodeProductPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoNodeProduct (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoNodeId($this->katao_node_id);

		$copyObj->setKataoPeriodId($this->katao_period_id);

		$copyObj->setKataoProductId($this->katao_product_id);


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoNodeProduct Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoNodeProductPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoNodeProductPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoNode object.
	 *
	 * @param      KataoNode $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoNode($v)
	{


		if ($v === null) {
			$this->setKataoNodeId(NULL);
		} else {
			$this->setKataoNodeId($v->getId());
		}


		$this->aKataoNode = $v;
	}


	/**
	 * Get the associated KataoNode object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoNode The associated KataoNode object.
	 * @throws     PropelException
	 */
	public function getKataoNode($con = null)
	{
		if ($this->aKataoNode === null && ($this->katao_node_id !== null)) {
			// include the related Peer class
			$this->aKataoNode = KataoNodePeer::retrieveByPK($this->katao_node_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoNodePeer::retrieveByPK($this->katao_node_id, $con);
			   $obj->addKataoNodes($this);
			 */
		}
		return $this->aKataoNode;
	}

	/**
	 * Declares an association between this object and a KataoPeriod object.
	 *
	 * @param      KataoPeriod $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoPeriod($v)
	{


		if ($v === null) {
			$this->setKataoPeriodId(NULL);
		} else {
			$this->setKataoPeriodId($v->getId());
		}


		$this->aKataoPeriod = $v;
	}


	/**
	 * Get the associated KataoPeriod object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoPeriod The associated KataoPeriod object.
	 * @throws     PropelException
	 */
	public function getKataoPeriod($con = null)
	{
		if ($this->aKataoPeriod === null && ($this->katao_period_id !== null)) {
			// include the related Peer class
			$this->aKataoPeriod = KataoPeriodPeer::retrieveByPK($this->katao_period_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoPeriodPeer::retrieveByPK($this->katao_period_id, $con);
			   $obj->addKataoPeriods($this);
			 */
		}
		return $this->aKataoPeriod;
	}

	/**
	 * Declares an association between this object and a KataoProduct object.
	 *
	 * @param      KataoProduct $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoProduct($v)
	{


		if ($v === null) {
			$this->setKataoProductId(NULL);
		} else {
			$this->setKataoProductId($v->getId());
		}


		$this->aKataoProduct = $v;
	}


	/**
	 * Get the associated KataoProduct object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoProduct The associated KataoProduct object.
	 * @throws     PropelException
	 */
	public function getKataoProduct($con = null)
	{
		if ($this->aKataoProduct === null && ($this->katao_product_id !== null)) {
			// include the related Peer class
			$this->aKataoProduct = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);
			   $obj->addKataoProducts($this);
			 */
		}
		return $this->aKataoProduct;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoNodeProduct:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoNodeProduct::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoNodeProduct
