<?php

/**
 * Base class that represents a row from the 'katao_order_product' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoOrderProduct extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoOrderProductPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_order_id field.
	 * @var        int
	 */
	protected $katao_order_id;


	/**
	 * The value for the katao_product_id field.
	 * @var        int
	 */
	protected $katao_product_id;


	/**
	 * The value for the product_reference field.
	 * @var        string
	 */
	protected $product_reference;


	/**
	 * The value for the product_name field.
	 * @var        string
	 */
	protected $product_name;


	/**
	 * The value for the product_price_euro field.
	 * @var        double
	 */
	protected $product_price_euro;


	/**
	 * The value for the product_margin field.
	 * @var        double
	 */
	protected $product_margin;


	/**
	 * The value for the product_tva_rate field.
	 * @var        double
	 */
	protected $product_tva_rate = 0;


	/**
	 * The value for the quantity field.
	 * @var        int
	 */
	protected $quantity = 1;


	/**
	 * The value for the quantity_received field.
	 * @var        int
	 */
	protected $quantity_received = 0;


	/**
	 * The value for the quantity_delivered field.
	 * @var        int
	 */
	protected $quantity_delivered = 0;


	/**
	 * The value for the has_been_received field.
	 * @var        int
	 */
	protected $has_been_received = 0;


	/**
	 * The value for the has_been_delivered field.
	 * @var        int
	 */
	protected $has_been_delivered = 0;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoOrder
	 */
	protected $aKataoOrder;

	/**
	 * @var        KataoProduct
	 */
	protected $aKataoProduct;

	/**
	 * Collection to store aggregation of collKataoCartProducts.
	 * @var        array
	 */
	protected $collKataoCartProducts;

	/**
	 * The criteria used to select the current contents of collKataoCartProducts.
	 * @var        Criteria
	 */
	protected $lastKataoCartProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoInvoiceProducts.
	 * @var        array
	 */
	protected $collKataoInvoiceProducts;

	/**
	 * The criteria used to select the current contents of collKataoInvoiceProducts.
	 * @var        Criteria
	 */
	protected $lastKataoInvoiceProductCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_order_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoOrderId()
	{

		return $this->katao_order_id;
	}

	/**
	 * Get the [katao_product_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoProductId()
	{

		return $this->katao_product_id;
	}

	/**
	 * Get the [product_reference] column value.
	 * 
	 * @return     string
	 */
	public function getProductReference()
	{

		return $this->product_reference;
	}

	/**
	 * Get the [product_name] column value.
	 * 
	 * @return     string
	 */
	public function getProductName()
	{

		return $this->product_name;
	}

	/**
	 * Get the [product_price_euro] column value.
	 * 
	 * @return     double
	 */
	public function getProductPriceEuro()
	{

		return $this->product_price_euro;
	}

	/**
	 * Get the [product_margin] column value.
	 * 
	 * @return     double
	 */
	public function getProductMargin()
	{

		return $this->product_margin;
	}

	/**
	 * Get the [product_tva_rate] column value.
	 * 
	 * @return     double
	 */
	public function getProductTvaRate()
	{

		return $this->product_tva_rate;
	}

	/**
	 * Get the [quantity] column value.
	 * 
	 * @return     int
	 */
	public function getQuantity()
	{

		return $this->quantity;
	}

	/**
	 * Get the [quantity_received] column value.
	 * 
	 * @return     int
	 */
	public function getQuantityReceived()
	{

		return $this->quantity_received;
	}

	/**
	 * Get the [quantity_delivered] column value.
	 * 
	 * @return     int
	 */
	public function getQuantityDelivered()
	{

		return $this->quantity_delivered;
	}

	/**
	 * Get the [has_been_received] column value.
	 * 
	 * @return     int
	 */
	public function getHasBeenReceived()
	{

		return $this->has_been_received;
	}

	/**
	 * Get the [has_been_delivered] column value.
	 * 
	 * @return     int
	 */
	public function getHasBeenDelivered()
	{

		return $this->has_been_delivered;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_order_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoOrderId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_order_id !== $v) {
			$this->katao_order_id = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::KATAO_ORDER_ID;
		}

		if ($this->aKataoOrder !== null && $this->aKataoOrder->getId() !== $v) {
			$this->aKataoOrder = null;
		}

	} // setKataoOrderId()

	/**
	 * Set the value of [katao_product_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoProductId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_product_id !== $v) {
			$this->katao_product_id = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::KATAO_PRODUCT_ID;
		}

		if ($this->aKataoProduct !== null && $this->aKataoProduct->getId() !== $v) {
			$this->aKataoProduct = null;
		}

	} // setKataoProductId()

	/**
	 * Set the value of [product_reference] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setProductReference($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->product_reference !== $v) {
			$this->product_reference = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::PRODUCT_REFERENCE;
		}

	} // setProductReference()

	/**
	 * Set the value of [product_name] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setProductName($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->product_name !== $v) {
			$this->product_name = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::PRODUCT_NAME;
		}

	} // setProductName()

	/**
	 * Set the value of [product_price_euro] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductPriceEuro($v)
	{

		if ($this->product_price_euro !== $v) {
			$this->product_price_euro = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::PRODUCT_PRICE_EURO;
		}

	} // setProductPriceEuro()

	/**
	 * Set the value of [product_margin] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductMargin($v)
	{

		if ($this->product_margin !== $v) {
			$this->product_margin = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::PRODUCT_MARGIN;
		}

	} // setProductMargin()

	/**
	 * Set the value of [product_tva_rate] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductTvaRate($v)
	{

		if ($this->product_tva_rate !== $v || $v === 0) {
			$this->product_tva_rate = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::PRODUCT_TVA_RATE;
		}

	} // setProductTvaRate()

	/**
	 * Set the value of [quantity] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setQuantity($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->quantity !== $v || $v === 1) {
			$this->quantity = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::QUANTITY;
		}

	} // setQuantity()

	/**
	 * Set the value of [quantity_received] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setQuantityReceived($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->quantity_received !== $v || $v === 0) {
			$this->quantity_received = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::QUANTITY_RECEIVED;
		}

	} // setQuantityReceived()

	/**
	 * Set the value of [quantity_delivered] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setQuantityDelivered($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->quantity_delivered !== $v || $v === 0) {
			$this->quantity_delivered = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::QUANTITY_DELIVERED;
		}

	} // setQuantityDelivered()

	/**
	 * Set the value of [has_been_received] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setHasBeenReceived($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->has_been_received !== $v || $v === 0) {
			$this->has_been_received = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::HAS_BEEN_RECEIVED;
		}

	} // setHasBeenReceived()

	/**
	 * Set the value of [has_been_delivered] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setHasBeenDelivered($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->has_been_delivered !== $v || $v === 0) {
			$this->has_been_delivered = $v;
			$this->modifiedColumns[] = KataoOrderProductPeer::HAS_BEEN_DELIVERED;
		}

	} // setHasBeenDelivered()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoOrderProductPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoOrderProductPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_order_id = $rs->getInt($startcol + 1);

			$this->katao_product_id = $rs->getInt($startcol + 2);

			$this->product_reference = $rs->getString($startcol + 3);

			$this->product_name = $rs->getString($startcol + 4);

			$this->product_price_euro = $rs->getFloat($startcol + 5);

			$this->product_margin = $rs->getFloat($startcol + 6);

			$this->product_tva_rate = $rs->getFloat($startcol + 7);

			$this->quantity = $rs->getInt($startcol + 8);

			$this->quantity_received = $rs->getInt($startcol + 9);

			$this->quantity_delivered = $rs->getInt($startcol + 10);

			$this->has_been_received = $rs->getInt($startcol + 11);

			$this->has_been_delivered = $rs->getInt($startcol + 12);

			$this->created_at = $rs->getTimestamp($startcol + 13, null);

			$this->updated_at = $rs->getTimestamp($startcol + 14, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 15; // 15 = KataoOrderProductPeer::NUM_COLUMNS - KataoOrderProductPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoOrderProduct object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoOrderProduct:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoOrderProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoOrderProductPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoOrderProduct:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoOrderProduct:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoOrderProductPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoOrderProductPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoOrderProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoOrderProduct:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoOrder !== null) {
				if ($this->aKataoOrder->isModified()) {
					$affectedRows += $this->aKataoOrder->save($con);
				}
				$this->setKataoOrder($this->aKataoOrder);
			}

			if ($this->aKataoProduct !== null) {
				if ($this->aKataoProduct->isModified()) {
					$affectedRows += $this->aKataoProduct->save($con);
				}
				$this->setKataoProduct($this->aKataoProduct);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoOrderProductPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoOrderProductPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoCartProducts !== null) {
				foreach($this->collKataoCartProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoInvoiceProducts !== null) {
				foreach($this->collKataoInvoiceProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoOrder !== null) {
				if (!$this->aKataoOrder->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoOrder->getValidationFailures());
				}
			}

			if ($this->aKataoProduct !== null) {
				if (!$this->aKataoProduct->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoProduct->getValidationFailures());
				}
			}


			if (($retval = KataoOrderProductPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoCartProducts !== null) {
					foreach($this->collKataoCartProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoInvoiceProducts !== null) {
					foreach($this->collKataoInvoiceProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoOrderProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoOrderId();
				break;
			case 2:
				return $this->getKataoProductId();
				break;
			case 3:
				return $this->getProductReference();
				break;
			case 4:
				return $this->getProductName();
				break;
			case 5:
				return $this->getProductPriceEuro();
				break;
			case 6:
				return $this->getProductMargin();
				break;
			case 7:
				return $this->getProductTvaRate();
				break;
			case 8:
				return $this->getQuantity();
				break;
			case 9:
				return $this->getQuantityReceived();
				break;
			case 10:
				return $this->getQuantityDelivered();
				break;
			case 11:
				return $this->getHasBeenReceived();
				break;
			case 12:
				return $this->getHasBeenDelivered();
				break;
			case 13:
				return $this->getCreatedAt();
				break;
			case 14:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoOrderProductPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoOrderId(),
			$keys[2] => $this->getKataoProductId(),
			$keys[3] => $this->getProductReference(),
			$keys[4] => $this->getProductName(),
			$keys[5] => $this->getProductPriceEuro(),
			$keys[6] => $this->getProductMargin(),
			$keys[7] => $this->getProductTvaRate(),
			$keys[8] => $this->getQuantity(),
			$keys[9] => $this->getQuantityReceived(),
			$keys[10] => $this->getQuantityDelivered(),
			$keys[11] => $this->getHasBeenReceived(),
			$keys[12] => $this->getHasBeenDelivered(),
			$keys[13] => $this->getCreatedAt(),
			$keys[14] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoOrderProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoOrderId($value);
				break;
			case 2:
				$this->setKataoProductId($value);
				break;
			case 3:
				$this->setProductReference($value);
				break;
			case 4:
				$this->setProductName($value);
				break;
			case 5:
				$this->setProductPriceEuro($value);
				break;
			case 6:
				$this->setProductMargin($value);
				break;
			case 7:
				$this->setProductTvaRate($value);
				break;
			case 8:
				$this->setQuantity($value);
				break;
			case 9:
				$this->setQuantityReceived($value);
				break;
			case 10:
				$this->setQuantityDelivered($value);
				break;
			case 11:
				$this->setHasBeenReceived($value);
				break;
			case 12:
				$this->setHasBeenDelivered($value);
				break;
			case 13:
				$this->setCreatedAt($value);
				break;
			case 14:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoOrderProductPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoOrderId($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setKataoProductId($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setProductReference($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setProductName($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setProductPriceEuro($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setProductMargin($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setProductTvaRate($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setQuantity($arr[$keys[8]]);
		if (array_key_exists($keys[9], $arr)) $this->setQuantityReceived($arr[$keys[9]]);
		if (array_key_exists($keys[10], $arr)) $this->setQuantityDelivered($arr[$keys[10]]);
		if (array_key_exists($keys[11], $arr)) $this->setHasBeenReceived($arr[$keys[11]]);
		if (array_key_exists($keys[12], $arr)) $this->setHasBeenDelivered($arr[$keys[12]]);
		if (array_key_exists($keys[13], $arr)) $this->setCreatedAt($arr[$keys[13]]);
		if (array_key_exists($keys[14], $arr)) $this->setUpdatedAt($arr[$keys[14]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoOrderProductPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoOrderProductPeer::ID)) $criteria->add(KataoOrderProductPeer::ID, $this->id);
		if ($this->isColumnModified(KataoOrderProductPeer::KATAO_ORDER_ID)) $criteria->add(KataoOrderProductPeer::KATAO_ORDER_ID, $this->katao_order_id);
		if ($this->isColumnModified(KataoOrderProductPeer::KATAO_PRODUCT_ID)) $criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->katao_product_id);
		if ($this->isColumnModified(KataoOrderProductPeer::PRODUCT_REFERENCE)) $criteria->add(KataoOrderProductPeer::PRODUCT_REFERENCE, $this->product_reference);
		if ($this->isColumnModified(KataoOrderProductPeer::PRODUCT_NAME)) $criteria->add(KataoOrderProductPeer::PRODUCT_NAME, $this->product_name);
		if ($this->isColumnModified(KataoOrderProductPeer::PRODUCT_PRICE_EURO)) $criteria->add(KataoOrderProductPeer::PRODUCT_PRICE_EURO, $this->product_price_euro);
		if ($this->isColumnModified(KataoOrderProductPeer::PRODUCT_MARGIN)) $criteria->add(KataoOrderProductPeer::PRODUCT_MARGIN, $this->product_margin);
		if ($this->isColumnModified(KataoOrderProductPeer::PRODUCT_TVA_RATE)) $criteria->add(KataoOrderProductPeer::PRODUCT_TVA_RATE, $this->product_tva_rate);
		if ($this->isColumnModified(KataoOrderProductPeer::QUANTITY)) $criteria->add(KataoOrderProductPeer::QUANTITY, $this->quantity);
		if ($this->isColumnModified(KataoOrderProductPeer::QUANTITY_RECEIVED)) $criteria->add(KataoOrderProductPeer::QUANTITY_RECEIVED, $this->quantity_received);
		if ($this->isColumnModified(KataoOrderProductPeer::QUANTITY_DELIVERED)) $criteria->add(KataoOrderProductPeer::QUANTITY_DELIVERED, $this->quantity_delivered);
		if ($this->isColumnModified(KataoOrderProductPeer::HAS_BEEN_RECEIVED)) $criteria->add(KataoOrderProductPeer::HAS_BEEN_RECEIVED, $this->has_been_received);
		if ($this->isColumnModified(KataoOrderProductPeer::HAS_BEEN_DELIVERED)) $criteria->add(KataoOrderProductPeer::HAS_BEEN_DELIVERED, $this->has_been_delivered);
		if ($this->isColumnModified(KataoOrderProductPeer::CREATED_AT)) $criteria->add(KataoOrderProductPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoOrderProductPeer::UPDATED_AT)) $criteria->add(KataoOrderProductPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoOrderProductPeer::DATABASE_NAME);

		$criteria->add(KataoOrderProductPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoOrderProduct (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoOrderId($this->katao_order_id);

		$copyObj->setKataoProductId($this->katao_product_id);

		$copyObj->setProductReference($this->product_reference);

		$copyObj->setProductName($this->product_name);

		$copyObj->setProductPriceEuro($this->product_price_euro);

		$copyObj->setProductMargin($this->product_margin);

		$copyObj->setProductTvaRate($this->product_tva_rate);

		$copyObj->setQuantity($this->quantity);

		$copyObj->setQuantityReceived($this->quantity_received);

		$copyObj->setQuantityDelivered($this->quantity_delivered);

		$copyObj->setHasBeenReceived($this->has_been_received);

		$copyObj->setHasBeenDelivered($this->has_been_delivered);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoCartProducts() as $relObj) {
				$copyObj->addKataoCartProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoInvoiceProducts() as $relObj) {
				$copyObj->addKataoInvoiceProduct($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoOrderProduct Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoOrderProductPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoOrderProductPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoOrder object.
	 *
	 * @param      KataoOrder $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoOrder($v)
	{


		if ($v === null) {
			$this->setKataoOrderId(NULL);
		} else {
			$this->setKataoOrderId($v->getId());
		}


		$this->aKataoOrder = $v;
	}


	/**
	 * Get the associated KataoOrder object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoOrder The associated KataoOrder object.
	 * @throws     PropelException
	 */
	public function getKataoOrder($con = null)
	{
		if ($this->aKataoOrder === null && ($this->katao_order_id !== null)) {
			// include the related Peer class
			$this->aKataoOrder = KataoOrderPeer::retrieveByPK($this->katao_order_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoOrderPeer::retrieveByPK($this->katao_order_id, $con);
			   $obj->addKataoOrders($this);
			 */
		}
		return $this->aKataoOrder;
	}

	/**
	 * Declares an association between this object and a KataoProduct object.
	 *
	 * @param      KataoProduct $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoProduct($v)
	{


		if ($v === null) {
			$this->setKataoProductId(NULL);
		} else {
			$this->setKataoProductId($v->getId());
		}


		$this->aKataoProduct = $v;
	}


	/**
	 * Get the associated KataoProduct object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoProduct The associated KataoProduct object.
	 * @throws     PropelException
	 */
	public function getKataoProduct($con = null)
	{
		if ($this->aKataoProduct === null && ($this->katao_product_id !== null)) {
			// include the related Peer class
			$this->aKataoProduct = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);
			   $obj->addKataoProducts($this);
			 */
		}
		return $this->aKataoProduct;
	}

	/**
	 * Temporary storage of collKataoCartProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoCartProducts()
	{
		if ($this->collKataoCartProducts === null) {
			$this->collKataoCartProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 * If this KataoOrderProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoCartProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
					$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;
		return $this->collKataoCartProducts;
	}

	/**
	 * Returns the number of related KataoCartProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoCartProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

		return KataoCartProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoCartProduct object to this object
	 * through the KataoCartProduct foreign key attribute
	 *
	 * @param      KataoCartProduct $l KataoCartProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoCartProduct(KataoCartProduct $l)
	{
		$this->collKataoCartProducts[] = $l;
		$l->setKataoOrderProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct is new, it will return
	 * an empty collection; or if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoOrderProduct.
	 */
	public function getKataoCartProductsJoinKataoCart($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoCart($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoCart($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct is new, it will return
	 * an empty collection; or if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoOrderProduct.
	 */
	public function getKataoCartProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}

	/**
	 * Temporary storage of collKataoInvoiceProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoInvoiceProducts()
	{
		if ($this->collKataoInvoiceProducts === null) {
			$this->collKataoInvoiceProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 * If this KataoOrderProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoInvoiceProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				KataoInvoiceProductPeer::addSelectColumns($criteria);
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				KataoInvoiceProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
					$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;
		return $this->collKataoInvoiceProducts;
	}

	/**
	 * Returns the number of related KataoInvoiceProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoInvoiceProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

		return KataoInvoiceProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoInvoiceProduct object to this object
	 * through the KataoInvoiceProduct foreign key attribute
	 *
	 * @param      KataoInvoiceProduct $l KataoInvoiceProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoInvoiceProduct(KataoInvoiceProduct $l)
	{
		$this->collKataoInvoiceProducts[] = $l;
		$l->setKataoOrderProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct is new, it will return
	 * an empty collection; or if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoOrderProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoInvoice($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoInvoice($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoInvoice($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct is new, it will return
	 * an empty collection; or if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoOrderProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoOrderProduct is new, it will return
	 * an empty collection; or if this KataoOrderProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoOrderProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoCartProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoCartProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_ORDER_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoCartProduct($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoOrderProduct:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoOrderProduct::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoOrderProduct
