<?php

/**
 * Base class that represents a row from the 'katao_period' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoPeriod extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoPeriodPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the name field.
	 * @var        string
	 */
	protected $name;


	/**
	 * The value for the begin_at field.
	 * @var        int
	 */
	protected $begin_at;


	/**
	 * The value for the finish_at field.
	 * @var        int
	 */
	protected $finish_at;


	/**
	 * The value for the order_ended_at field.
	 * @var        int
	 */
	protected $order_ended_at;


	/**
	 * The value for the status field.
	 * @var        int
	 */
	protected $status = 1;


	/**
	 * The value for the exported field.
	 * @var        int
	 */
	protected $exported = 0;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * Collection to store aggregation of collKataoNodeProducts.
	 * @var        array
	 */
	protected $collKataoNodeProducts;

	/**
	 * The criteria used to select the current contents of collKataoNodeProducts.
	 * @var        Criteria
	 */
	protected $lastKataoNodeProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoNodeSuppliers.
	 * @var        array
	 */
	protected $collKataoNodeSuppliers;

	/**
	 * The criteria used to select the current contents of collKataoNodeSuppliers.
	 * @var        Criteria
	 */
	protected $lastKataoNodeSupplierCriteria = null;

	/**
	 * Collection to store aggregation of collKataoPeriodProducts.
	 * @var        array
	 */
	protected $collKataoPeriodProducts;

	/**
	 * The criteria used to select the current contents of collKataoPeriodProducts.
	 * @var        Criteria
	 */
	protected $lastKataoPeriodProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoPeriodSuppliers.
	 * @var        array
	 */
	protected $collKataoPeriodSuppliers;

	/**
	 * The criteria used to select the current contents of collKataoPeriodSuppliers.
	 * @var        Criteria
	 */
	protected $lastKataoPeriodSupplierCriteria = null;

	/**
	 * Collection to store aggregation of collKataoCarts.
	 * @var        array
	 */
	protected $collKataoCarts;

	/**
	 * The criteria used to select the current contents of collKataoCarts.
	 * @var        Criteria
	 */
	protected $lastKataoCartCriteria = null;

	/**
	 * Collection to store aggregation of collKataoOrders.
	 * @var        array
	 */
	protected $collKataoOrders;

	/**
	 * The criteria used to select the current contents of collKataoOrders.
	 * @var        Criteria
	 */
	protected $lastKataoOrderCriteria = null;

	/**
	 * Collection to store aggregation of collKataoInvoices.
	 * @var        array
	 */
	protected $collKataoInvoices;

	/**
	 * The criteria used to select the current contents of collKataoInvoices.
	 * @var        Criteria
	 */
	protected $lastKataoInvoiceCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierInvoices.
	 * @var        array
	 */
	protected $collKataoSupplierInvoices;

	/**
	 * The criteria used to select the current contents of collKataoSupplierInvoices.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierInvoiceCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [name] column value.
	 * 
	 * @return     string
	 */
	public function getName()
	{

		return $this->name;
	}

	/**
	 * Get the [optionally formatted] [begin_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getBeginAt($format = 'Y-m-d H:i:s')
	{

		if ($this->begin_at === null || $this->begin_at === '') {
			return null;
		} elseif (!is_int($this->begin_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->begin_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [begin_at] as date/time value: " . var_export($this->begin_at, true));
			}
		} else {
			$ts = $this->begin_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [finish_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getFinishAt($format = 'Y-m-d H:i:s')
	{

		if ($this->finish_at === null || $this->finish_at === '') {
			return null;
		} elseif (!is_int($this->finish_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->finish_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [finish_at] as date/time value: " . var_export($this->finish_at, true));
			}
		} else {
			$ts = $this->finish_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [order_ended_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getOrderEndedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->order_ended_at === null || $this->order_ended_at === '') {
			return null;
		} elseif (!is_int($this->order_ended_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->order_ended_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [order_ended_at] as date/time value: " . var_export($this->order_ended_at, true));
			}
		} else {
			$ts = $this->order_ended_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [status] column value.
	 * 
	 * @return     int
	 */
	public function getStatus()
	{

		return $this->status;
	}

	/**
	 * Get the [exported] column value.
	 * 
	 * @return     int
	 */
	public function getExported()
	{

		return $this->exported;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoPeriodPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [name] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setName($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->name !== $v) {
			$this->name = $v;
			$this->modifiedColumns[] = KataoPeriodPeer::NAME;
		}

	} // setName()

	/**
	 * Set the value of [begin_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setBeginAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [begin_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->begin_at !== $ts) {
			$this->begin_at = $ts;
			$this->modifiedColumns[] = KataoPeriodPeer::BEGIN_AT;
		}

	} // setBeginAt()

	/**
	 * Set the value of [finish_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setFinishAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [finish_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->finish_at !== $ts) {
			$this->finish_at = $ts;
			$this->modifiedColumns[] = KataoPeriodPeer::FINISH_AT;
		}

	} // setFinishAt()

	/**
	 * Set the value of [order_ended_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setOrderEndedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [order_ended_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->order_ended_at !== $ts) {
			$this->order_ended_at = $ts;
			$this->modifiedColumns[] = KataoPeriodPeer::ORDER_ENDED_AT;
		}

	} // setOrderEndedAt()

	/**
	 * Set the value of [status] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setStatus($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->status !== $v || $v === 1) {
			$this->status = $v;
			$this->modifiedColumns[] = KataoPeriodPeer::STATUS;
		}

	} // setStatus()

	/**
	 * Set the value of [exported] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setExported($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->exported !== $v || $v === 0) {
			$this->exported = $v;
			$this->modifiedColumns[] = KataoPeriodPeer::EXPORTED;
		}

	} // setExported()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoPeriodPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoPeriodPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->name = $rs->getString($startcol + 1);

			$this->begin_at = $rs->getTimestamp($startcol + 2, null);

			$this->finish_at = $rs->getTimestamp($startcol + 3, null);

			$this->order_ended_at = $rs->getTimestamp($startcol + 4, null);

			$this->status = $rs->getInt($startcol + 5);

			$this->exported = $rs->getInt($startcol + 6);

			$this->created_at = $rs->getTimestamp($startcol + 7, null);

			$this->updated_at = $rs->getTimestamp($startcol + 8, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 9; // 9 = KataoPeriodPeer::NUM_COLUMNS - KataoPeriodPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoPeriod object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoPeriod:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoPeriodPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoPeriodPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoPeriod:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoPeriod:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoPeriodPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoPeriodPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoPeriodPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoPeriod:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoPeriodPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoPeriodPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoNodeProducts !== null) {
				foreach($this->collKataoNodeProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoNodeSuppliers !== null) {
				foreach($this->collKataoNodeSuppliers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoPeriodProducts !== null) {
				foreach($this->collKataoPeriodProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoPeriodSuppliers !== null) {
				foreach($this->collKataoPeriodSuppliers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoCarts !== null) {
				foreach($this->collKataoCarts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoOrders !== null) {
				foreach($this->collKataoOrders as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoInvoices !== null) {
				foreach($this->collKataoInvoices as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierInvoices !== null) {
				foreach($this->collKataoSupplierInvoices as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			if (($retval = KataoPeriodPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoNodeProducts !== null) {
					foreach($this->collKataoNodeProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoNodeSuppliers !== null) {
					foreach($this->collKataoNodeSuppliers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoPeriodProducts !== null) {
					foreach($this->collKataoPeriodProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoPeriodSuppliers !== null) {
					foreach($this->collKataoPeriodSuppliers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoCarts !== null) {
					foreach($this->collKataoCarts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoOrders !== null) {
					foreach($this->collKataoOrders as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoInvoices !== null) {
					foreach($this->collKataoInvoices as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierInvoices !== null) {
					foreach($this->collKataoSupplierInvoices as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoPeriodPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getName();
				break;
			case 2:
				return $this->getBeginAt();
				break;
			case 3:
				return $this->getFinishAt();
				break;
			case 4:
				return $this->getOrderEndedAt();
				break;
			case 5:
				return $this->getStatus();
				break;
			case 6:
				return $this->getExported();
				break;
			case 7:
				return $this->getCreatedAt();
				break;
			case 8:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoPeriodPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getName(),
			$keys[2] => $this->getBeginAt(),
			$keys[3] => $this->getFinishAt(),
			$keys[4] => $this->getOrderEndedAt(),
			$keys[5] => $this->getStatus(),
			$keys[6] => $this->getExported(),
			$keys[7] => $this->getCreatedAt(),
			$keys[8] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoPeriodPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setName($value);
				break;
			case 2:
				$this->setBeginAt($value);
				break;
			case 3:
				$this->setFinishAt($value);
				break;
			case 4:
				$this->setOrderEndedAt($value);
				break;
			case 5:
				$this->setStatus($value);
				break;
			case 6:
				$this->setExported($value);
				break;
			case 7:
				$this->setCreatedAt($value);
				break;
			case 8:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoPeriodPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setName($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setBeginAt($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setFinishAt($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setOrderEndedAt($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setStatus($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setExported($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setCreatedAt($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setUpdatedAt($arr[$keys[8]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoPeriodPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoPeriodPeer::ID)) $criteria->add(KataoPeriodPeer::ID, $this->id);
		if ($this->isColumnModified(KataoPeriodPeer::NAME)) $criteria->add(KataoPeriodPeer::NAME, $this->name);
		if ($this->isColumnModified(KataoPeriodPeer::BEGIN_AT)) $criteria->add(KataoPeriodPeer::BEGIN_AT, $this->begin_at);
		if ($this->isColumnModified(KataoPeriodPeer::FINISH_AT)) $criteria->add(KataoPeriodPeer::FINISH_AT, $this->finish_at);
		if ($this->isColumnModified(KataoPeriodPeer::ORDER_ENDED_AT)) $criteria->add(KataoPeriodPeer::ORDER_ENDED_AT, $this->order_ended_at);
		if ($this->isColumnModified(KataoPeriodPeer::STATUS)) $criteria->add(KataoPeriodPeer::STATUS, $this->status);
		if ($this->isColumnModified(KataoPeriodPeer::EXPORTED)) $criteria->add(KataoPeriodPeer::EXPORTED, $this->exported);
		if ($this->isColumnModified(KataoPeriodPeer::CREATED_AT)) $criteria->add(KataoPeriodPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoPeriodPeer::UPDATED_AT)) $criteria->add(KataoPeriodPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoPeriodPeer::DATABASE_NAME);

		$criteria->add(KataoPeriodPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoPeriod (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setName($this->name);

		$copyObj->setBeginAt($this->begin_at);

		$copyObj->setFinishAt($this->finish_at);

		$copyObj->setOrderEndedAt($this->order_ended_at);

		$copyObj->setStatus($this->status);

		$copyObj->setExported($this->exported);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoNodeProducts() as $relObj) {
				$copyObj->addKataoNodeProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoNodeSuppliers() as $relObj) {
				$copyObj->addKataoNodeSupplier($relObj->copy($deepCopy));
			}

			foreach($this->getKataoPeriodProducts() as $relObj) {
				$copyObj->addKataoPeriodProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoPeriodSuppliers() as $relObj) {
				$copyObj->addKataoPeriodSupplier($relObj->copy($deepCopy));
			}

			foreach($this->getKataoCarts() as $relObj) {
				$copyObj->addKataoCart($relObj->copy($deepCopy));
			}

			foreach($this->getKataoOrders() as $relObj) {
				$copyObj->addKataoOrder($relObj->copy($deepCopy));
			}

			foreach($this->getKataoInvoices() as $relObj) {
				$copyObj->addKataoInvoice($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierInvoices() as $relObj) {
				$copyObj->addKataoSupplierInvoice($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoPeriod Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoPeriodPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoPeriodPeer();
		}
		return self::$peer;
	}

	/**
	 * Temporary storage of collKataoNodeProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeProducts()
	{
		if ($this->collKataoNodeProducts === null) {
			$this->collKataoNodeProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
					$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;
		return $this->collKataoNodeProducts;
	}

	/**
	 * Returns the number of related KataoNodeProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoNodeProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeProduct object to this object
	 * through the KataoNodeProduct foreign key attribute
	 *
	 * @param      KataoNodeProduct $l KataoNodeProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeProduct(KataoNodeProduct $l)
	{
		$this->collKataoNodeProducts[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoNodeProductsJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoNodeProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}

	/**
	 * Temporary storage of collKataoNodeSuppliers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeSuppliers()
	{
		if ($this->collKataoNodeSuppliers === null) {
			$this->collKataoNodeSuppliers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeSuppliers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
					$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;
		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Returns the number of related KataoNodeSuppliers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeSuppliers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoNodeSupplierPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeSupplier object to this object
	 * through the KataoNodeSupplier foreign key attribute
	 *
	 * @param      KataoNodeSupplier $l KataoNodeSupplier
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeSupplier(KataoNodeSupplier $l)
	{
		$this->collKataoNodeSuppliers[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoNodeSuppliersJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoNodeSuppliersJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Temporary storage of collKataoPeriodProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoPeriodProducts()
	{
		if ($this->collKataoPeriodProducts === null) {
			$this->collKataoPeriodProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoPeriodProducts from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoPeriodProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoPeriodProducts = array();
			} else {

				$criteria->add(KataoPeriodProductPeer::KATAO_PERIOD_ID, $this->getId());

				KataoPeriodProductPeer::addSelectColumns($criteria);
				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoPeriodProductPeer::KATAO_PERIOD_ID, $this->getId());

				KataoPeriodProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoPeriodProductCriteria) || !$this->lastKataoPeriodProductCriteria->equals($criteria)) {
					$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoPeriodProductCriteria = $criteria;
		return $this->collKataoPeriodProducts;
	}

	/**
	 * Returns the number of related KataoPeriodProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoPeriodProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoPeriodProductPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoPeriodProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoPeriodProduct object to this object
	 * through the KataoPeriodProduct foreign key attribute
	 *
	 * @param      KataoPeriodProduct $l KataoPeriodProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoPeriodProduct(KataoPeriodProduct $l)
	{
		$this->collKataoPeriodProducts[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoPeriodProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoPeriodProductsJoinKataoProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodProducts === null) {
			if ($this->isNew()) {
				$this->collKataoPeriodProducts = array();
			} else {

				$criteria->add(KataoPeriodProductPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoPeriodProductPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoPeriodProductCriteria) || !$this->lastKataoPeriodProductCriteria->equals($criteria)) {
				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelectJoinKataoProduct($criteria, $con);
			}
		}
		$this->lastKataoPeriodProductCriteria = $criteria;

		return $this->collKataoPeriodProducts;
	}

	/**
	 * Temporary storage of collKataoPeriodSuppliers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoPeriodSuppliers()
	{
		if ($this->collKataoPeriodSuppliers === null) {
			$this->collKataoPeriodSuppliers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoPeriodSuppliers from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoPeriodSuppliers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodSuppliers === null) {
			if ($this->isNew()) {
			   $this->collKataoPeriodSuppliers = array();
			} else {

				$criteria->add(KataoPeriodSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				KataoPeriodSupplierPeer::addSelectColumns($criteria);
				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoPeriodSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				KataoPeriodSupplierPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoPeriodSupplierCriteria) || !$this->lastKataoPeriodSupplierCriteria->equals($criteria)) {
					$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoPeriodSupplierCriteria = $criteria;
		return $this->collKataoPeriodSuppliers;
	}

	/**
	 * Returns the number of related KataoPeriodSuppliers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoPeriodSuppliers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoPeriodSupplierPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoPeriodSupplierPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoPeriodSupplier object to this object
	 * through the KataoPeriodSupplier foreign key attribute
	 *
	 * @param      KataoPeriodSupplier $l KataoPeriodSupplier
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoPeriodSupplier(KataoPeriodSupplier $l)
	{
		$this->collKataoPeriodSuppliers[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoPeriodSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoPeriodSuppliersJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoPeriodSuppliers = array();
			} else {

				$criteria->add(KataoPeriodSupplierPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoPeriodSupplierPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoPeriodSupplierCriteria) || !$this->lastKataoPeriodSupplierCriteria->equals($criteria)) {
				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoPeriodSupplierCriteria = $criteria;

		return $this->collKataoPeriodSuppliers;
	}

	/**
	 * Temporary storage of collKataoCarts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoCarts()
	{
		if ($this->collKataoCarts === null) {
			$this->collKataoCarts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoCarts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
			   $this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

				KataoCartPeer::addSelectColumns($criteria);
				$this->collKataoCarts = KataoCartPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

				KataoCartPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
					$this->collKataoCarts = KataoCartPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoCartCriteria = $criteria;
		return $this->collKataoCarts;
	}

	/**
	 * Returns the number of related KataoCarts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoCarts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoCartPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoCart object to this object
	 * through the KataoCart foreign key attribute
	 *
	 * @param      KataoCart $l KataoCart
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoCart(KataoCart $l)
	{
		$this->collKataoCarts[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoCartsJoinKataoUser($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
				$this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoUser($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoUser($criteria, $con);
			}
		}
		$this->lastKataoCartCriteria = $criteria;

		return $this->collKataoCarts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoCarts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoCartsJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCarts === null) {
			if ($this->isNew()) {
				$this->collKataoCarts = array();
			} else {

				$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoCartCriteria) || !$this->lastKataoCartCriteria->equals($criteria)) {
				$this->collKataoCarts = KataoCartPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoCartCriteria = $criteria;

		return $this->collKataoCarts;
	}

	/**
	 * Temporary storage of collKataoOrders to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoOrders()
	{
		if ($this->collKataoOrders === null) {
			$this->collKataoOrders = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoOrders($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
			   $this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
					$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoOrderCriteria = $criteria;
		return $this->collKataoOrders;
	}

	/**
	 * Returns the number of related KataoOrders.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoOrders($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

		return KataoOrderPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoOrder object to this object
	 * through the KataoOrder foreign key attribute
	 *
	 * @param      KataoOrder $l KataoOrder
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoOrder(KataoOrder $l)
	{
		$this->collKataoOrders[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoOrdersJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoOrdersJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}

	/**
	 * Temporary storage of collKataoInvoices to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoInvoices()
	{
		if ($this->collKataoInvoices === null) {
			$this->collKataoInvoices = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoInvoices($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
			   $this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				KataoInvoicePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
					$this->collKataoInvoices = KataoInvoicePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;
		return $this->collKataoInvoices;
	}

	/**
	 * Returns the number of related KataoInvoices.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoInvoices($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

		return KataoInvoicePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoInvoice object to this object
	 * through the KataoInvoice foreign key attribute
	 *
	 * @param      KataoInvoice $l KataoInvoice
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoInvoice(KataoInvoice $l)
	{
		$this->collKataoInvoices[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoInvoicesJoinKataoMember($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoMember($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoInvoicesJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoInvoicesJoinKataoCart($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoInvoices = array();
			} else {

				$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoCart($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoicePeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceCriteria) || !$this->lastKataoInvoiceCriteria->equals($criteria)) {
				$this->collKataoInvoices = KataoInvoicePeer::doSelectJoinKataoCart($criteria, $con);
			}
		}
		$this->lastKataoInvoiceCriteria = $criteria;

		return $this->collKataoInvoices;
	}

	/**
	 * Temporary storage of collKataoSupplierInvoices to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierInvoices()
	{
		if ($this->collKataoSupplierInvoices === null) {
			$this->collKataoSupplierInvoices = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoSupplierInvoices from storage.
	 * If this KataoPeriod is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierInvoices($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoices === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierInvoices = array();
			} else {

				$criteria->add(KataoSupplierInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				KataoSupplierInvoicePeer::addSelectColumns($criteria);
				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				KataoSupplierInvoicePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierInvoiceCriteria) || !$this->lastKataoSupplierInvoiceCriteria->equals($criteria)) {
					$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierInvoiceCriteria = $criteria;
		return $this->collKataoSupplierInvoices;
	}

	/**
	 * Returns the number of related KataoSupplierInvoices.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierInvoices($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierInvoicePeer::KATAO_PERIOD_ID, $this->getId());

		return KataoSupplierInvoicePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierInvoice object to this object
	 * through the KataoSupplierInvoice foreign key attribute
	 *
	 * @param      KataoSupplierInvoice $l KataoSupplierInvoice
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierInvoice(KataoSupplierInvoice $l)
	{
		$this->collKataoSupplierInvoices[] = $l;
		$l->setKataoPeriod($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoPeriod is new, it will return
	 * an empty collection; or if this KataoPeriod has previously
	 * been saved, it will retrieve related KataoSupplierInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoPeriod.
	 */
	public function getKataoSupplierInvoicesJoinKataoSupplier($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierInvoices = array();
			} else {

				$criteria->add(KataoSupplierInvoicePeer::KATAO_PERIOD_ID, $this->getId());

				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierInvoicePeer::KATAO_PERIOD_ID, $this->getId());

			if (!isset($this->lastKataoSupplierInvoiceCriteria) || !$this->lastKataoSupplierInvoiceCriteria->equals($criteria)) {
				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelectJoinKataoSupplier($criteria, $con);
			}
		}
		$this->lastKataoSupplierInvoiceCriteria = $criteria;

		return $this->collKataoSupplierInvoices;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoPeriod:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoPeriod::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoPeriod
