<?php

/**
 * Base class that represents a row from the 'katao_product' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoProduct extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoProductPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the reference field.
	 * @var        string
	 */
	protected $reference;


	/**
	 * The value for the name field.
	 * @var        string
	 */
	protected $name;


	/**
	 * The value for the url_identifier field.
	 * @var        string
	 */
	protected $url_identifier;


	/**
	 * The value for the description field.
	 * @var        string
	 */
	protected $description;


	/**
	 * The value for the katao_product_family_id field.
	 * @var        int
	 */
	protected $katao_product_family_id;


	/**
	 * The value for the katao_supplier_id field.
	 * @var        int
	 */
	protected $katao_supplier_id;


	/**
	 * The value for the max_sol_amount field.
	 * @var        int
	 */
	protected $max_sol_amount;


	/**
	 * The value for the unit_price_euro field.
	 * @var        double
	 */
	protected $unit_price_euro;


	/**
	 * The value for the margin field.
	 * @var        double
	 */
	protected $margin;


	/**
	 * The value for the tva_rate field.
	 * @var        double
	 */
	protected $tva_rate = 0;


	/**
	 * The value for the min_order_number field.
	 * @var        int
	 */
	protected $min_order_number = 1;


	/**
	 * The value for the picture field.
	 * @var        string
	 */
	protected $picture;


	/**
	 * The value for the delivery_delay field.
	 * @var        int
	 */
	protected $delivery_delay;


	/**
	 * The value for the is_archived field.
	 * @var        int
	 */
	protected $is_archived = 0;


	/**
	 * The value for the accounting_code_purchase field.
	 * @var        string
	 */
	protected $accounting_code_purchase;


	/**
	 * The value for the accounting_code_sell field.
	 * @var        string
	 */
	protected $accounting_code_sell;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoProductFamily
	 */
	protected $aKataoProductFamily;

	/**
	 * @var        KataoSupplier
	 */
	protected $aKataoSupplier;

	/**
	 * Collection to store aggregation of collKataoNodeProducts.
	 * @var        array
	 */
	protected $collKataoNodeProducts;

	/**
	 * The criteria used to select the current contents of collKataoNodeProducts.
	 * @var        Criteria
	 */
	protected $lastKataoNodeProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoPeriodProducts.
	 * @var        array
	 */
	protected $collKataoPeriodProducts;

	/**
	 * The criteria used to select the current contents of collKataoPeriodProducts.
	 * @var        Criteria
	 */
	protected $lastKataoPeriodProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoCartProducts.
	 * @var        array
	 */
	protected $collKataoCartProducts;

	/**
	 * The criteria used to select the current contents of collKataoCartProducts.
	 * @var        Criteria
	 */
	protected $lastKataoCartProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoOrderProducts.
	 * @var        array
	 */
	protected $collKataoOrderProducts;

	/**
	 * The criteria used to select the current contents of collKataoOrderProducts.
	 * @var        Criteria
	 */
	protected $lastKataoOrderProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoInvoiceProducts.
	 * @var        array
	 */
	protected $collKataoInvoiceProducts;

	/**
	 * The criteria used to select the current contents of collKataoInvoiceProducts.
	 * @var        Criteria
	 */
	protected $lastKataoInvoiceProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierInvoiceProducts.
	 * @var        array
	 */
	protected $collKataoSupplierInvoiceProducts;

	/**
	 * The criteria used to select the current contents of collKataoSupplierInvoiceProducts.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierInvoiceProductCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [reference] column value.
	 * 
	 * @return     string
	 */
	public function getReference()
	{

		return $this->reference;
	}

	/**
	 * Get the [name] column value.
	 * 
	 * @return     string
	 */
	public function getName()
	{

		return $this->name;
	}

	/**
	 * Get the [url_identifier] column value.
	 * 
	 * @return     string
	 */
	public function getUrlIdentifier()
	{

		return $this->url_identifier;
	}

	/**
	 * Get the [description] column value.
	 * 
	 * @return     string
	 */
	public function getDescription()
	{

		return $this->description;
	}

	/**
	 * Get the [katao_product_family_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoProductFamilyId()
	{

		return $this->katao_product_family_id;
	}

	/**
	 * Get the [katao_supplier_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoSupplierId()
	{

		return $this->katao_supplier_id;
	}

	/**
	 * Get the [max_sol_amount] column value.
	 * 
	 * @return     int
	 */
	public function getMaxSolAmount()
	{

		return $this->max_sol_amount;
	}

	/**
	 * Get the [unit_price_euro] column value.
	 * 
	 * @return     double
	 */
	public function getUnitPriceEuro()
	{

		return $this->unit_price_euro;
	}

	/**
	 * Get the [margin] column value.
	 * 
	 * @return     double
	 */
	public function getMargin()
	{

		return $this->margin;
	}

	/**
	 * Get the [tva_rate] column value.
	 * 
	 * @return     double
	 */
	public function getTvaRate()
	{

		return $this->tva_rate;
	}

	/**
	 * Get the [min_order_number] column value.
	 * 
	 * @return     int
	 */
	public function getMinOrderNumber()
	{

		return $this->min_order_number;
	}

	/**
	 * Get the [picture] column value.
	 * 
	 * @return     string
	 */
	public function getPicture()
	{

		return $this->picture;
	}

	/**
	 * Get the [delivery_delay] column value.
	 * 
	 * @return     int
	 */
	public function getDeliveryDelay()
	{

		return $this->delivery_delay;
	}

	/**
	 * Get the [is_archived] column value.
	 * 
	 * @return     int
	 */
	public function getIsArchived()
	{

		return $this->is_archived;
	}

	/**
	 * Get the [accounting_code_purchase] column value.
	 * 
	 * @return     string
	 */
	public function getAccountingCodePurchase()
	{

		return $this->accounting_code_purchase;
	}

	/**
	 * Get the [accounting_code_sell] column value.
	 * 
	 * @return     string
	 */
	public function getAccountingCodeSell()
	{

		return $this->accounting_code_sell;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoProductPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [reference] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setReference($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->reference !== $v) {
			$this->reference = $v;
			$this->modifiedColumns[] = KataoProductPeer::REFERENCE;
		}

	} // setReference()

	/**
	 * Set the value of [name] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setName($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->name !== $v) {
			$this->name = $v;
			$this->modifiedColumns[] = KataoProductPeer::NAME;
		}

	} // setName()

	/**
	 * Set the value of [url_identifier] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setUrlIdentifier($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->url_identifier !== $v) {
			$this->url_identifier = $v;
			$this->modifiedColumns[] = KataoProductPeer::URL_IDENTIFIER;
		}

	} // setUrlIdentifier()

	/**
	 * Set the value of [description] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setDescription($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->description !== $v) {
			$this->description = $v;
			$this->modifiedColumns[] = KataoProductPeer::DESCRIPTION;
		}

	} // setDescription()

	/**
	 * Set the value of [katao_product_family_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoProductFamilyId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_product_family_id !== $v) {
			$this->katao_product_family_id = $v;
			$this->modifiedColumns[] = KataoProductPeer::KATAO_PRODUCT_FAMILY_ID;
		}

		if ($this->aKataoProductFamily !== null && $this->aKataoProductFamily->getId() !== $v) {
			$this->aKataoProductFamily = null;
		}

	} // setKataoProductFamilyId()

	/**
	 * Set the value of [katao_supplier_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoSupplierId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_supplier_id !== $v) {
			$this->katao_supplier_id = $v;
			$this->modifiedColumns[] = KataoProductPeer::KATAO_SUPPLIER_ID;
		}

		if ($this->aKataoSupplier !== null && $this->aKataoSupplier->getId() !== $v) {
			$this->aKataoSupplier = null;
		}

	} // setKataoSupplierId()

	/**
	 * Set the value of [max_sol_amount] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setMaxSolAmount($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->max_sol_amount !== $v) {
			$this->max_sol_amount = $v;
			$this->modifiedColumns[] = KataoProductPeer::MAX_SOL_AMOUNT;
		}

	} // setMaxSolAmount()

	/**
	 * Set the value of [unit_price_euro] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setUnitPriceEuro($v)
	{

		if ($this->unit_price_euro !== $v) {
			$this->unit_price_euro = $v;
			$this->modifiedColumns[] = KataoProductPeer::UNIT_PRICE_EURO;
		}

	} // setUnitPriceEuro()

	/**
	 * Set the value of [margin] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setMargin($v)
	{

		if ($this->margin !== $v) {
			$this->margin = $v;
			$this->modifiedColumns[] = KataoProductPeer::MARGIN;
		}

	} // setMargin()

	/**
	 * Set the value of [tva_rate] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setTvaRate($v)
	{

		if ($this->tva_rate !== $v || $v === 0) {
			$this->tva_rate = $v;
			$this->modifiedColumns[] = KataoProductPeer::TVA_RATE;
		}

	} // setTvaRate()

	/**
	 * Set the value of [min_order_number] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setMinOrderNumber($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->min_order_number !== $v || $v === 1) {
			$this->min_order_number = $v;
			$this->modifiedColumns[] = KataoProductPeer::MIN_ORDER_NUMBER;
		}

	} // setMinOrderNumber()

	/**
	 * Set the value of [picture] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setPicture($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->picture !== $v) {
			$this->picture = $v;
			$this->modifiedColumns[] = KataoProductPeer::PICTURE;
		}

	} // setPicture()

	/**
	 * Set the value of [delivery_delay] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setDeliveryDelay($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->delivery_delay !== $v) {
			$this->delivery_delay = $v;
			$this->modifiedColumns[] = KataoProductPeer::DELIVERY_DELAY;
		}

	} // setDeliveryDelay()

	/**
	 * Set the value of [is_archived] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setIsArchived($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->is_archived !== $v || $v === 0) {
			$this->is_archived = $v;
			$this->modifiedColumns[] = KataoProductPeer::IS_ARCHIVED;
		}

	} // setIsArchived()

	/**
	 * Set the value of [accounting_code_purchase] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAccountingCodePurchase($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->accounting_code_purchase !== $v) {
			$this->accounting_code_purchase = $v;
			$this->modifiedColumns[] = KataoProductPeer::ACCOUNTING_CODE_PURCHASE;
		}

	} // setAccountingCodePurchase()

	/**
	 * Set the value of [accounting_code_sell] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAccountingCodeSell($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->accounting_code_sell !== $v) {
			$this->accounting_code_sell = $v;
			$this->modifiedColumns[] = KataoProductPeer::ACCOUNTING_CODE_SELL;
		}

	} // setAccountingCodeSell()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoProductPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoProductPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->reference = $rs->getString($startcol + 1);

			$this->name = $rs->getString($startcol + 2);

			$this->url_identifier = $rs->getString($startcol + 3);

			$this->description = $rs->getString($startcol + 4);

			$this->katao_product_family_id = $rs->getInt($startcol + 5);

			$this->katao_supplier_id = $rs->getInt($startcol + 6);

			$this->max_sol_amount = $rs->getInt($startcol + 7);

			$this->unit_price_euro = $rs->getFloat($startcol + 8);

			$this->margin = $rs->getFloat($startcol + 9);

			$this->tva_rate = $rs->getFloat($startcol + 10);

			$this->min_order_number = $rs->getInt($startcol + 11);

			$this->picture = $rs->getString($startcol + 12);

			$this->delivery_delay = $rs->getInt($startcol + 13);

			$this->is_archived = $rs->getInt($startcol + 14);

			$this->accounting_code_purchase = $rs->getString($startcol + 15);

			$this->accounting_code_sell = $rs->getString($startcol + 16);

			$this->created_at = $rs->getTimestamp($startcol + 17, null);

			$this->updated_at = $rs->getTimestamp($startcol + 18, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 19; // 19 = KataoProductPeer::NUM_COLUMNS - KataoProductPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoProduct object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoProduct:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoProductPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoProduct:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoProduct:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoProductPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoProductPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoProduct:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoProductFamily !== null) {
				if ($this->aKataoProductFamily->isModified()) {
					$affectedRows += $this->aKataoProductFamily->save($con);
				}
				$this->setKataoProductFamily($this->aKataoProductFamily);
			}

			if ($this->aKataoSupplier !== null) {
				if ($this->aKataoSupplier->isModified()) {
					$affectedRows += $this->aKataoSupplier->save($con);
				}
				$this->setKataoSupplier($this->aKataoSupplier);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoProductPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoProductPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoNodeProducts !== null) {
				foreach($this->collKataoNodeProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoPeriodProducts !== null) {
				foreach($this->collKataoPeriodProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoCartProducts !== null) {
				foreach($this->collKataoCartProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoOrderProducts !== null) {
				foreach($this->collKataoOrderProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoInvoiceProducts !== null) {
				foreach($this->collKataoInvoiceProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierInvoiceProducts !== null) {
				foreach($this->collKataoSupplierInvoiceProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoProductFamily !== null) {
				if (!$this->aKataoProductFamily->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoProductFamily->getValidationFailures());
				}
			}

			if ($this->aKataoSupplier !== null) {
				if (!$this->aKataoSupplier->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoSupplier->getValidationFailures());
				}
			}


			if (($retval = KataoProductPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoNodeProducts !== null) {
					foreach($this->collKataoNodeProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoPeriodProducts !== null) {
					foreach($this->collKataoPeriodProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoCartProducts !== null) {
					foreach($this->collKataoCartProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoOrderProducts !== null) {
					foreach($this->collKataoOrderProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoInvoiceProducts !== null) {
					foreach($this->collKataoInvoiceProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierInvoiceProducts !== null) {
					foreach($this->collKataoSupplierInvoiceProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getReference();
				break;
			case 2:
				return $this->getName();
				break;
			case 3:
				return $this->getUrlIdentifier();
				break;
			case 4:
				return $this->getDescription();
				break;
			case 5:
				return $this->getKataoProductFamilyId();
				break;
			case 6:
				return $this->getKataoSupplierId();
				break;
			case 7:
				return $this->getMaxSolAmount();
				break;
			case 8:
				return $this->getUnitPriceEuro();
				break;
			case 9:
				return $this->getMargin();
				break;
			case 10:
				return $this->getTvaRate();
				break;
			case 11:
				return $this->getMinOrderNumber();
				break;
			case 12:
				return $this->getPicture();
				break;
			case 13:
				return $this->getDeliveryDelay();
				break;
			case 14:
				return $this->getIsArchived();
				break;
			case 15:
				return $this->getAccountingCodePurchase();
				break;
			case 16:
				return $this->getAccountingCodeSell();
				break;
			case 17:
				return $this->getCreatedAt();
				break;
			case 18:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoProductPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getReference(),
			$keys[2] => $this->getName(),
			$keys[3] => $this->getUrlIdentifier(),
			$keys[4] => $this->getDescription(),
			$keys[5] => $this->getKataoProductFamilyId(),
			$keys[6] => $this->getKataoSupplierId(),
			$keys[7] => $this->getMaxSolAmount(),
			$keys[8] => $this->getUnitPriceEuro(),
			$keys[9] => $this->getMargin(),
			$keys[10] => $this->getTvaRate(),
			$keys[11] => $this->getMinOrderNumber(),
			$keys[12] => $this->getPicture(),
			$keys[13] => $this->getDeliveryDelay(),
			$keys[14] => $this->getIsArchived(),
			$keys[15] => $this->getAccountingCodePurchase(),
			$keys[16] => $this->getAccountingCodeSell(),
			$keys[17] => $this->getCreatedAt(),
			$keys[18] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setReference($value);
				break;
			case 2:
				$this->setName($value);
				break;
			case 3:
				$this->setUrlIdentifier($value);
				break;
			case 4:
				$this->setDescription($value);
				break;
			case 5:
				$this->setKataoProductFamilyId($value);
				break;
			case 6:
				$this->setKataoSupplierId($value);
				break;
			case 7:
				$this->setMaxSolAmount($value);
				break;
			case 8:
				$this->setUnitPriceEuro($value);
				break;
			case 9:
				$this->setMargin($value);
				break;
			case 10:
				$this->setTvaRate($value);
				break;
			case 11:
				$this->setMinOrderNumber($value);
				break;
			case 12:
				$this->setPicture($value);
				break;
			case 13:
				$this->setDeliveryDelay($value);
				break;
			case 14:
				$this->setIsArchived($value);
				break;
			case 15:
				$this->setAccountingCodePurchase($value);
				break;
			case 16:
				$this->setAccountingCodeSell($value);
				break;
			case 17:
				$this->setCreatedAt($value);
				break;
			case 18:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoProductPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setReference($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setName($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setUrlIdentifier($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setDescription($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setKataoProductFamilyId($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setKataoSupplierId($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setMaxSolAmount($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setUnitPriceEuro($arr[$keys[8]]);
		if (array_key_exists($keys[9], $arr)) $this->setMargin($arr[$keys[9]]);
		if (array_key_exists($keys[10], $arr)) $this->setTvaRate($arr[$keys[10]]);
		if (array_key_exists($keys[11], $arr)) $this->setMinOrderNumber($arr[$keys[11]]);
		if (array_key_exists($keys[12], $arr)) $this->setPicture($arr[$keys[12]]);
		if (array_key_exists($keys[13], $arr)) $this->setDeliveryDelay($arr[$keys[13]]);
		if (array_key_exists($keys[14], $arr)) $this->setIsArchived($arr[$keys[14]]);
		if (array_key_exists($keys[15], $arr)) $this->setAccountingCodePurchase($arr[$keys[15]]);
		if (array_key_exists($keys[16], $arr)) $this->setAccountingCodeSell($arr[$keys[16]]);
		if (array_key_exists($keys[17], $arr)) $this->setCreatedAt($arr[$keys[17]]);
		if (array_key_exists($keys[18], $arr)) $this->setUpdatedAt($arr[$keys[18]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoProductPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoProductPeer::ID)) $criteria->add(KataoProductPeer::ID, $this->id);
		if ($this->isColumnModified(KataoProductPeer::REFERENCE)) $criteria->add(KataoProductPeer::REFERENCE, $this->reference);
		if ($this->isColumnModified(KataoProductPeer::NAME)) $criteria->add(KataoProductPeer::NAME, $this->name);
		if ($this->isColumnModified(KataoProductPeer::URL_IDENTIFIER)) $criteria->add(KataoProductPeer::URL_IDENTIFIER, $this->url_identifier);
		if ($this->isColumnModified(KataoProductPeer::DESCRIPTION)) $criteria->add(KataoProductPeer::DESCRIPTION, $this->description);
		if ($this->isColumnModified(KataoProductPeer::KATAO_PRODUCT_FAMILY_ID)) $criteria->add(KataoProductPeer::KATAO_PRODUCT_FAMILY_ID, $this->katao_product_family_id);
		if ($this->isColumnModified(KataoProductPeer::KATAO_SUPPLIER_ID)) $criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->katao_supplier_id);
		if ($this->isColumnModified(KataoProductPeer::MAX_SOL_AMOUNT)) $criteria->add(KataoProductPeer::MAX_SOL_AMOUNT, $this->max_sol_amount);
		if ($this->isColumnModified(KataoProductPeer::UNIT_PRICE_EURO)) $criteria->add(KataoProductPeer::UNIT_PRICE_EURO, $this->unit_price_euro);
		if ($this->isColumnModified(KataoProductPeer::MARGIN)) $criteria->add(KataoProductPeer::MARGIN, $this->margin);
		if ($this->isColumnModified(KataoProductPeer::TVA_RATE)) $criteria->add(KataoProductPeer::TVA_RATE, $this->tva_rate);
		if ($this->isColumnModified(KataoProductPeer::MIN_ORDER_NUMBER)) $criteria->add(KataoProductPeer::MIN_ORDER_NUMBER, $this->min_order_number);
		if ($this->isColumnModified(KataoProductPeer::PICTURE)) $criteria->add(KataoProductPeer::PICTURE, $this->picture);
		if ($this->isColumnModified(KataoProductPeer::DELIVERY_DELAY)) $criteria->add(KataoProductPeer::DELIVERY_DELAY, $this->delivery_delay);
		if ($this->isColumnModified(KataoProductPeer::IS_ARCHIVED)) $criteria->add(KataoProductPeer::IS_ARCHIVED, $this->is_archived);
		if ($this->isColumnModified(KataoProductPeer::ACCOUNTING_CODE_PURCHASE)) $criteria->add(KataoProductPeer::ACCOUNTING_CODE_PURCHASE, $this->accounting_code_purchase);
		if ($this->isColumnModified(KataoProductPeer::ACCOUNTING_CODE_SELL)) $criteria->add(KataoProductPeer::ACCOUNTING_CODE_SELL, $this->accounting_code_sell);
		if ($this->isColumnModified(KataoProductPeer::CREATED_AT)) $criteria->add(KataoProductPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoProductPeer::UPDATED_AT)) $criteria->add(KataoProductPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoProductPeer::DATABASE_NAME);

		$criteria->add(KataoProductPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoProduct (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setReference($this->reference);

		$copyObj->setName($this->name);

		$copyObj->setUrlIdentifier($this->url_identifier);

		$copyObj->setDescription($this->description);

		$copyObj->setKataoProductFamilyId($this->katao_product_family_id);

		$copyObj->setKataoSupplierId($this->katao_supplier_id);

		$copyObj->setMaxSolAmount($this->max_sol_amount);

		$copyObj->setUnitPriceEuro($this->unit_price_euro);

		$copyObj->setMargin($this->margin);

		$copyObj->setTvaRate($this->tva_rate);

		$copyObj->setMinOrderNumber($this->min_order_number);

		$copyObj->setPicture($this->picture);

		$copyObj->setDeliveryDelay($this->delivery_delay);

		$copyObj->setIsArchived($this->is_archived);

		$copyObj->setAccountingCodePurchase($this->accounting_code_purchase);

		$copyObj->setAccountingCodeSell($this->accounting_code_sell);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoNodeProducts() as $relObj) {
				$copyObj->addKataoNodeProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoPeriodProducts() as $relObj) {
				$copyObj->addKataoPeriodProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoCartProducts() as $relObj) {
				$copyObj->addKataoCartProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoOrderProducts() as $relObj) {
				$copyObj->addKataoOrderProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoInvoiceProducts() as $relObj) {
				$copyObj->addKataoInvoiceProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierInvoiceProducts() as $relObj) {
				$copyObj->addKataoSupplierInvoiceProduct($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoProduct Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoProductPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoProductPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoProductFamily object.
	 *
	 * @param      KataoProductFamily $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoProductFamily($v)
	{


		if ($v === null) {
			$this->setKataoProductFamilyId(NULL);
		} else {
			$this->setKataoProductFamilyId($v->getId());
		}


		$this->aKataoProductFamily = $v;
	}


	/**
	 * Get the associated KataoProductFamily object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoProductFamily The associated KataoProductFamily object.
	 * @throws     PropelException
	 */
	public function getKataoProductFamily($con = null)
	{
		if ($this->aKataoProductFamily === null && ($this->katao_product_family_id !== null)) {
			// include the related Peer class
			$this->aKataoProductFamily = KataoProductFamilyPeer::retrieveByPK($this->katao_product_family_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoProductFamilyPeer::retrieveByPK($this->katao_product_family_id, $con);
			   $obj->addKataoProductFamilys($this);
			 */
		}
		return $this->aKataoProductFamily;
	}

	/**
	 * Declares an association between this object and a KataoSupplier object.
	 *
	 * @param      KataoSupplier $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoSupplier($v)
	{


		if ($v === null) {
			$this->setKataoSupplierId(NULL);
		} else {
			$this->setKataoSupplierId($v->getId());
		}


		$this->aKataoSupplier = $v;
	}


	/**
	 * Get the associated KataoSupplier object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoSupplier The associated KataoSupplier object.
	 * @throws     PropelException
	 */
	public function getKataoSupplier($con = null)
	{
		if ($this->aKataoSupplier === null && ($this->katao_supplier_id !== null)) {
			// include the related Peer class
			$this->aKataoSupplier = KataoSupplierPeer::retrieveByPK($this->katao_supplier_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoSupplierPeer::retrieveByPK($this->katao_supplier_id, $con);
			   $obj->addKataoSuppliers($this);
			 */
		}
		return $this->aKataoSupplier;
	}

	/**
	 * Temporary storage of collKataoNodeProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeProducts()
	{
		if ($this->collKataoNodeProducts === null) {
			$this->collKataoNodeProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoNodeProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
					$this->collKataoNodeProducts = KataoNodeProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;
		return $this->collKataoNodeProducts;
	}

	/**
	 * Returns the number of related KataoNodeProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoNodeProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeProduct object to this object
	 * through the KataoNodeProduct foreign key attribute
	 *
	 * @param      KataoNodeProduct $l KataoNodeProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeProduct(KataoNodeProduct $l)
	{
		$this->collKataoNodeProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoNodeProductsJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoNodeProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoNodeProductsJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeProducts === null) {
			if ($this->isNew()) {
				$this->collKataoNodeProducts = array();
			} else {

				$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoNodeProductCriteria) || !$this->lastKataoNodeProductCriteria->equals($criteria)) {
				$this->collKataoNodeProducts = KataoNodeProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoNodeProductCriteria = $criteria;

		return $this->collKataoNodeProducts;
	}

	/**
	 * Temporary storage of collKataoPeriodProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoPeriodProducts()
	{
		if ($this->collKataoPeriodProducts === null) {
			$this->collKataoPeriodProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoPeriodProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoPeriodProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoPeriodProducts = array();
			} else {

				$criteria->add(KataoPeriodProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoPeriodProductPeer::addSelectColumns($criteria);
				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoPeriodProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoPeriodProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoPeriodProductCriteria) || !$this->lastKataoPeriodProductCriteria->equals($criteria)) {
					$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoPeriodProductCriteria = $criteria;
		return $this->collKataoPeriodProducts;
	}

	/**
	 * Returns the number of related KataoPeriodProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoPeriodProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoPeriodProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoPeriodProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoPeriodProduct object to this object
	 * through the KataoPeriodProduct foreign key attribute
	 *
	 * @param      KataoPeriodProduct $l KataoPeriodProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoPeriodProduct(KataoPeriodProduct $l)
	{
		$this->collKataoPeriodProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoPeriodProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoPeriodProductsJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodProducts === null) {
			if ($this->isNew()) {
				$this->collKataoPeriodProducts = array();
			} else {

				$criteria->add(KataoPeriodProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoPeriodProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoPeriodProductCriteria) || !$this->lastKataoPeriodProductCriteria->equals($criteria)) {
				$this->collKataoPeriodProducts = KataoPeriodProductPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoPeriodProductCriteria = $criteria;

		return $this->collKataoPeriodProducts;
	}

	/**
	 * Temporary storage of collKataoCartProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoCartProducts()
	{
		if ($this->collKataoCartProducts === null) {
			$this->collKataoCartProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoCartProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoCartProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
					$this->collKataoCartProducts = KataoCartProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;
		return $this->collKataoCartProducts;
	}

	/**
	 * Returns the number of related KataoCartProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoCartProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoCartProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoCartProduct object to this object
	 * through the KataoCartProduct foreign key attribute
	 *
	 * @param      KataoCartProduct $l KataoCartProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoCartProduct(KataoCartProduct $l)
	{
		$this->collKataoCartProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoCartProductsJoinKataoCart($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoCart($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoCart($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoCartProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoCartProductsJoinKataoOrderProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoCartProducts === null) {
			if ($this->isNew()) {
				$this->collKataoCartProducts = array();
			} else {

				$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoCartProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoCartProductCriteria) || !$this->lastKataoCartProductCriteria->equals($criteria)) {
				$this->collKataoCartProducts = KataoCartProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		}
		$this->lastKataoCartProductCriteria = $criteria;

		return $this->collKataoCartProducts;
	}

	/**
	 * Temporary storage of collKataoOrderProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoOrderProducts()
	{
		if ($this->collKataoOrderProducts === null) {
			$this->collKataoOrderProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoOrderProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoOrderProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrderProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoOrderProducts = array();
			} else {

				$criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoOrderProductPeer::addSelectColumns($criteria);
				$this->collKataoOrderProducts = KataoOrderProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoOrderProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoOrderProductCriteria) || !$this->lastKataoOrderProductCriteria->equals($criteria)) {
					$this->collKataoOrderProducts = KataoOrderProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoOrderProductCriteria = $criteria;
		return $this->collKataoOrderProducts;
	}

	/**
	 * Returns the number of related KataoOrderProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoOrderProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoOrderProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoOrderProduct object to this object
	 * through the KataoOrderProduct foreign key attribute
	 *
	 * @param      KataoOrderProduct $l KataoOrderProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoOrderProduct(KataoOrderProduct $l)
	{
		$this->collKataoOrderProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoOrderProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoOrderProductsJoinKataoOrder($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrderProducts === null) {
			if ($this->isNew()) {
				$this->collKataoOrderProducts = array();
			} else {

				$criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoOrderProducts = KataoOrderProductPeer::doSelectJoinKataoOrder($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoOrderProductCriteria) || !$this->lastKataoOrderProductCriteria->equals($criteria)) {
				$this->collKataoOrderProducts = KataoOrderProductPeer::doSelectJoinKataoOrder($criteria, $con);
			}
		}
		$this->lastKataoOrderProductCriteria = $criteria;

		return $this->collKataoOrderProducts;
	}

	/**
	 * Temporary storage of collKataoInvoiceProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoInvoiceProducts()
	{
		if ($this->collKataoInvoiceProducts === null) {
			$this->collKataoInvoiceProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoInvoiceProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoInvoiceProductPeer::addSelectColumns($criteria);
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoInvoiceProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
					$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;
		return $this->collKataoInvoiceProducts;
	}

	/**
	 * Returns the number of related KataoInvoiceProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoInvoiceProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoInvoiceProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoInvoiceProduct object to this object
	 * through the KataoInvoiceProduct foreign key attribute
	 *
	 * @param      KataoInvoiceProduct $l KataoInvoiceProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoInvoiceProduct(KataoInvoiceProduct $l)
	{
		$this->collKataoInvoiceProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoInvoice($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoInvoice($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoInvoice($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoCartProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoCartProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoCartProduct($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoInvoiceProductsJoinKataoOrderProduct($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoInvoiceProducts = array();
			} else {

				$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoInvoiceProductCriteria) || !$this->lastKataoInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoInvoiceProducts = KataoInvoiceProductPeer::doSelectJoinKataoOrderProduct($criteria, $con);
			}
		}
		$this->lastKataoInvoiceProductCriteria = $criteria;

		return $this->collKataoInvoiceProducts;
	}

	/**
	 * Temporary storage of collKataoSupplierInvoiceProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierInvoiceProducts()
	{
		if ($this->collKataoSupplierInvoiceProducts === null) {
			$this->collKataoSupplierInvoiceProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct has previously
	 * been saved, it will retrieve related KataoSupplierInvoiceProducts from storage.
	 * If this KataoProduct is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierInvoiceProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoiceProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierInvoiceProducts = array();
			} else {

				$criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoSupplierInvoiceProductPeer::addSelectColumns($criteria);
				$this->collKataoSupplierInvoiceProducts = KataoSupplierInvoiceProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				KataoSupplierInvoiceProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierInvoiceProductCriteria) || !$this->lastKataoSupplierInvoiceProductCriteria->equals($criteria)) {
					$this->collKataoSupplierInvoiceProducts = KataoSupplierInvoiceProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierInvoiceProductCriteria = $criteria;
		return $this->collKataoSupplierInvoiceProducts;
	}

	/**
	 * Returns the number of related KataoSupplierInvoiceProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierInvoiceProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

		return KataoSupplierInvoiceProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierInvoiceProduct object to this object
	 * through the KataoSupplierInvoiceProduct foreign key attribute
	 *
	 * @param      KataoSupplierInvoiceProduct $l KataoSupplierInvoiceProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierInvoiceProduct(KataoSupplierInvoiceProduct $l)
	{
		$this->collKataoSupplierInvoiceProducts[] = $l;
		$l->setKataoProduct($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoProduct is new, it will return
	 * an empty collection; or if this KataoProduct has previously
	 * been saved, it will retrieve related KataoSupplierInvoiceProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoProduct.
	 */
	public function getKataoSupplierInvoiceProductsJoinKataoSupplierInvoice($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoiceProducts === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierInvoiceProducts = array();
			} else {

				$criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

				$this->collKataoSupplierInvoiceProducts = KataoSupplierInvoiceProductPeer::doSelectJoinKataoSupplierInvoice($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->getId());

			if (!isset($this->lastKataoSupplierInvoiceProductCriteria) || !$this->lastKataoSupplierInvoiceProductCriteria->equals($criteria)) {
				$this->collKataoSupplierInvoiceProducts = KataoSupplierInvoiceProductPeer::doSelectJoinKataoSupplierInvoice($criteria, $con);
			}
		}
		$this->lastKataoSupplierInvoiceProductCriteria = $criteria;

		return $this->collKataoSupplierInvoiceProducts;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoProduct:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoProduct::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoProduct
