<?php

/**
 * Base class that represents a row from the 'katao_supplier' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoSupplier extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoSupplierPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_index field.
	 * @var        int
	 */
	protected $katao_index;


	/**
	 * The value for the member_id field.
	 * @var        int
	 */
	protected $member_id;


	/**
	 * The value for the referer_id field.
	 * @var        int
	 */
	protected $referer_id;


	/**
	 * The value for the name field.
	 * @var        string
	 */
	protected $name;


	/**
	 * The value for the url_identifier field.
	 * @var        string
	 */
	protected $url_identifier;


	/**
	 * The value for the presentation field.
	 * @var        string
	 */
	protected $presentation;


	/**
	 * The value for the email field.
	 * @var        string
	 */
	protected $email;


	/**
	 * The value for the address1 field.
	 * @var        string
	 */
	protected $address1;


	/**
	 * The value for the address2 field.
	 * @var        string
	 */
	protected $address2;


	/**
	 * The value for the zip field.
	 * @var        string
	 */
	protected $zip;


	/**
	 * The value for the city field.
	 * @var        string
	 */
	protected $city;


	/**
	 * The value for the phone field.
	 * @var        string
	 */
	protected $phone;


	/**
	 * The value for the fax field.
	 * @var        string
	 */
	protected $fax;


	/**
	 * The value for the website field.
	 * @var        string
	 */
	protected $website;


	/**
	 * The value for the gmap_lng field.
	 * @var        double
	 */
	protected $gmap_lng;


	/**
	 * The value for the gmap_lat field.
	 * @var        double
	 */
	protected $gmap_lat;


	/**
	 * The value for the accounting_code field.
	 * @var        string
	 */
	protected $accounting_code;


	/**
	 * The value for the accounting_code_sol field.
	 * @var        string
	 */
	protected $accounting_code_sol;


	/**
	 * The value for the siret_number field.
	 * @var        string
	 */
	protected $siret_number;


	/**
	 * The value for the is_global field.
	 * @var        int
	 */
	protected $is_global = 0;


	/**
	 * The value for the order_min_amount field.
	 * @var        double
	 */
	protected $order_min_amount = 0;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoMember
	 */
	protected $aKataoMemberRelatedByMemberId;

	/**
	 * @var        KataoMember
	 */
	protected $aKataoMemberRelatedByRefererId;

	/**
	 * Collection to store aggregation of collKataoProducts.
	 * @var        array
	 */
	protected $collKataoProducts;

	/**
	 * The criteria used to select the current contents of collKataoProducts.
	 * @var        Criteria
	 */
	protected $lastKataoProductCriteria = null;

	/**
	 * Collection to store aggregation of collKataoUsers.
	 * @var        array
	 */
	protected $collKataoUsers;

	/**
	 * The criteria used to select the current contents of collKataoUsers.
	 * @var        Criteria
	 */
	protected $lastKataoUserCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierNodes.
	 * @var        array
	 */
	protected $collKataoSupplierNodes;

	/**
	 * The criteria used to select the current contents of collKataoSupplierNodes.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierNodeCriteria = null;

	/**
	 * Collection to store aggregation of collKataoNodeSuppliers.
	 * @var        array
	 */
	protected $collKataoNodeSuppliers;

	/**
	 * The criteria used to select the current contents of collKataoNodeSuppliers.
	 * @var        Criteria
	 */
	protected $lastKataoNodeSupplierCriteria = null;

	/**
	 * Collection to store aggregation of collKataoPeriodSuppliers.
	 * @var        array
	 */
	protected $collKataoPeriodSuppliers;

	/**
	 * The criteria used to select the current contents of collKataoPeriodSuppliers.
	 * @var        Criteria
	 */
	protected $lastKataoPeriodSupplierCriteria = null;

	/**
	 * Collection to store aggregation of collKataoOrders.
	 * @var        array
	 */
	protected $collKataoOrders;

	/**
	 * The criteria used to select the current contents of collKataoOrders.
	 * @var        Criteria
	 */
	protected $lastKataoOrderCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierInvoices.
	 * @var        array
	 */
	protected $collKataoSupplierInvoices;

	/**
	 * The criteria used to select the current contents of collKataoSupplierInvoices.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierInvoiceCriteria = null;

	/**
	 * Collection to store aggregation of collKataoSupplierProductCategorys.
	 * @var        array
	 */
	protected $collKataoSupplierProductCategorys;

	/**
	 * The criteria used to select the current contents of collKataoSupplierProductCategorys.
	 * @var        Criteria
	 */
	protected $lastKataoSupplierProductCategoryCriteria = null;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_index] column value.
	 * 
	 * @return     int
	 */
	public function getKataoIndex()
	{

		return $this->katao_index;
	}

	/**
	 * Get the [member_id] column value.
	 * 
	 * @return     int
	 */
	public function getMemberId()
	{

		return $this->member_id;
	}

	/**
	 * Get the [referer_id] column value.
	 * 
	 * @return     int
	 */
	public function getRefererId()
	{

		return $this->referer_id;
	}

	/**
	 * Get the [name] column value.
	 * 
	 * @return     string
	 */
	public function getName()
	{

		return $this->name;
	}

	/**
	 * Get the [url_identifier] column value.
	 * 
	 * @return     string
	 */
	public function getUrlIdentifier()
	{

		return $this->url_identifier;
	}

	/**
	 * Get the [presentation] column value.
	 * 
	 * @return     string
	 */
	public function getPresentation()
	{

		return $this->presentation;
	}

	/**
	 * Get the [email] column value.
	 * 
	 * @return     string
	 */
	public function getEmail()
	{

		return $this->email;
	}

	/**
	 * Get the [address1] column value.
	 * 
	 * @return     string
	 */
	public function getAddress1()
	{

		return $this->address1;
	}

	/**
	 * Get the [address2] column value.
	 * 
	 * @return     string
	 */
	public function getAddress2()
	{

		return $this->address2;
	}

	/**
	 * Get the [zip] column value.
	 * 
	 * @return     string
	 */
	public function getZip()
	{

		return $this->zip;
	}

	/**
	 * Get the [city] column value.
	 * 
	 * @return     string
	 */
	public function getCity()
	{

		return $this->city;
	}

	/**
	 * Get the [phone] column value.
	 * 
	 * @return     string
	 */
	public function getPhone()
	{

		return $this->phone;
	}

	/**
	 * Get the [fax] column value.
	 * 
	 * @return     string
	 */
	public function getFax()
	{

		return $this->fax;
	}

	/**
	 * Get the [website] column value.
	 * 
	 * @return     string
	 */
	public function getWebsite()
	{

		return $this->website;
	}

	/**
	 * Get the [gmap_lng] column value.
	 * 
	 * @return     double
	 */
	public function getGmapLng()
	{

		return $this->gmap_lng;
	}

	/**
	 * Get the [gmap_lat] column value.
	 * 
	 * @return     double
	 */
	public function getGmapLat()
	{

		return $this->gmap_lat;
	}

	/**
	 * Get the [accounting_code] column value.
	 * 
	 * @return     string
	 */
	public function getAccountingCode()
	{

		return $this->accounting_code;
	}

	/**
	 * Get the [accounting_code_sol] column value.
	 * 
	 * @return     string
	 */
	public function getAccountingCodeSol()
	{

		return $this->accounting_code_sol;
	}

	/**
	 * Get the [siret_number] column value.
	 * 
	 * @return     string
	 */
	public function getSiretNumber()
	{

		return $this->siret_number;
	}

	/**
	 * Get the [is_global] column value.
	 * 
	 * @return     int
	 */
	public function getIsGlobal()
	{

		return $this->is_global;
	}

	/**
	 * Get the [order_min_amount] column value.
	 * 
	 * @return     double
	 */
	public function getOrderMinAmount()
	{

		return $this->order_min_amount;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_index] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoIndex($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_index !== $v) {
			$this->katao_index = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::KATAO_INDEX;
		}

	} // setKataoIndex()

	/**
	 * Set the value of [member_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setMemberId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->member_id !== $v) {
			$this->member_id = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::MEMBER_ID;
		}

		if ($this->aKataoMemberRelatedByMemberId !== null && $this->aKataoMemberRelatedByMemberId->getId() !== $v) {
			$this->aKataoMemberRelatedByMemberId = null;
		}

	} // setMemberId()

	/**
	 * Set the value of [referer_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setRefererId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->referer_id !== $v) {
			$this->referer_id = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::REFERER_ID;
		}

		if ($this->aKataoMemberRelatedByRefererId !== null && $this->aKataoMemberRelatedByRefererId->getId() !== $v) {
			$this->aKataoMemberRelatedByRefererId = null;
		}

	} // setRefererId()

	/**
	 * Set the value of [name] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setName($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->name !== $v) {
			$this->name = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::NAME;
		}

	} // setName()

	/**
	 * Set the value of [url_identifier] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setUrlIdentifier($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->url_identifier !== $v) {
			$this->url_identifier = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::URL_IDENTIFIER;
		}

	} // setUrlIdentifier()

	/**
	 * Set the value of [presentation] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setPresentation($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->presentation !== $v) {
			$this->presentation = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::PRESENTATION;
		}

	} // setPresentation()

	/**
	 * Set the value of [email] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setEmail($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->email !== $v) {
			$this->email = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::EMAIL;
		}

	} // setEmail()

	/**
	 * Set the value of [address1] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAddress1($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->address1 !== $v) {
			$this->address1 = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ADDRESS1;
		}

	} // setAddress1()

	/**
	 * Set the value of [address2] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAddress2($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->address2 !== $v) {
			$this->address2 = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ADDRESS2;
		}

	} // setAddress2()

	/**
	 * Set the value of [zip] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setZip($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->zip !== $v) {
			$this->zip = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ZIP;
		}

	} // setZip()

	/**
	 * Set the value of [city] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setCity($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->city !== $v) {
			$this->city = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::CITY;
		}

	} // setCity()

	/**
	 * Set the value of [phone] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setPhone($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->phone !== $v) {
			$this->phone = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::PHONE;
		}

	} // setPhone()

	/**
	 * Set the value of [fax] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setFax($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->fax !== $v) {
			$this->fax = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::FAX;
		}

	} // setFax()

	/**
	 * Set the value of [website] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setWebsite($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->website !== $v) {
			$this->website = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::WEBSITE;
		}

	} // setWebsite()

	/**
	 * Set the value of [gmap_lng] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setGmapLng($v)
	{

		if ($this->gmap_lng !== $v) {
			$this->gmap_lng = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::GMAP_LNG;
		}

	} // setGmapLng()

	/**
	 * Set the value of [gmap_lat] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setGmapLat($v)
	{

		if ($this->gmap_lat !== $v) {
			$this->gmap_lat = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::GMAP_LAT;
		}

	} // setGmapLat()

	/**
	 * Set the value of [accounting_code] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAccountingCode($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->accounting_code !== $v) {
			$this->accounting_code = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ACCOUNTING_CODE;
		}

	} // setAccountingCode()

	/**
	 * Set the value of [accounting_code_sol] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setAccountingCodeSol($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->accounting_code_sol !== $v) {
			$this->accounting_code_sol = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ACCOUNTING_CODE_SOL;
		}

	} // setAccountingCodeSol()

	/**
	 * Set the value of [siret_number] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setSiretNumber($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->siret_number !== $v) {
			$this->siret_number = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::SIRET_NUMBER;
		}

	} // setSiretNumber()

	/**
	 * Set the value of [is_global] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setIsGlobal($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->is_global !== $v || $v === 0) {
			$this->is_global = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::IS_GLOBAL;
		}

	} // setIsGlobal()

	/**
	 * Set the value of [order_min_amount] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setOrderMinAmount($v)
	{

		if ($this->order_min_amount !== $v || $v === 0) {
			$this->order_min_amount = $v;
			$this->modifiedColumns[] = KataoSupplierPeer::ORDER_MIN_AMOUNT;
		}

	} // setOrderMinAmount()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoSupplierPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoSupplierPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_index = $rs->getInt($startcol + 1);

			$this->member_id = $rs->getInt($startcol + 2);

			$this->referer_id = $rs->getInt($startcol + 3);

			$this->name = $rs->getString($startcol + 4);

			$this->url_identifier = $rs->getString($startcol + 5);

			$this->presentation = $rs->getString($startcol + 6);

			$this->email = $rs->getString($startcol + 7);

			$this->address1 = $rs->getString($startcol + 8);

			$this->address2 = $rs->getString($startcol + 9);

			$this->zip = $rs->getString($startcol + 10);

			$this->city = $rs->getString($startcol + 11);

			$this->phone = $rs->getString($startcol + 12);

			$this->fax = $rs->getString($startcol + 13);

			$this->website = $rs->getString($startcol + 14);

			$this->gmap_lng = $rs->getFloat($startcol + 15);

			$this->gmap_lat = $rs->getFloat($startcol + 16);

			$this->accounting_code = $rs->getString($startcol + 17);

			$this->accounting_code_sol = $rs->getString($startcol + 18);

			$this->siret_number = $rs->getString($startcol + 19);

			$this->is_global = $rs->getInt($startcol + 20);

			$this->order_min_amount = $rs->getFloat($startcol + 21);

			$this->created_at = $rs->getTimestamp($startcol + 22, null);

			$this->updated_at = $rs->getTimestamp($startcol + 23, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 24; // 24 = KataoSupplierPeer::NUM_COLUMNS - KataoSupplierPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoSupplier object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoSupplier:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoSupplierPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoSupplierPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoSupplier:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoSupplier:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoSupplierPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoSupplierPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoSupplierPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoSupplier:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoMemberRelatedByMemberId !== null) {
				if ($this->aKataoMemberRelatedByMemberId->isModified()) {
					$affectedRows += $this->aKataoMemberRelatedByMemberId->save($con);
				}
				$this->setKataoMemberRelatedByMemberId($this->aKataoMemberRelatedByMemberId);
			}

			if ($this->aKataoMemberRelatedByRefererId !== null) {
				if ($this->aKataoMemberRelatedByRefererId->isModified()) {
					$affectedRows += $this->aKataoMemberRelatedByRefererId->save($con);
				}
				$this->setKataoMemberRelatedByRefererId($this->aKataoMemberRelatedByRefererId);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoSupplierPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoSupplierPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			if ($this->collKataoProducts !== null) {
				foreach($this->collKataoProducts as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoUsers !== null) {
				foreach($this->collKataoUsers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierNodes !== null) {
				foreach($this->collKataoSupplierNodes as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoNodeSuppliers !== null) {
				foreach($this->collKataoNodeSuppliers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoPeriodSuppliers !== null) {
				foreach($this->collKataoPeriodSuppliers as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoOrders !== null) {
				foreach($this->collKataoOrders as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierInvoices !== null) {
				foreach($this->collKataoSupplierInvoices as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			if ($this->collKataoSupplierProductCategorys !== null) {
				foreach($this->collKataoSupplierProductCategorys as $referrerFK) {
					if (!$referrerFK->isDeleted()) {
						$affectedRows += $referrerFK->save($con);
					}
				}
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoMemberRelatedByMemberId !== null) {
				if (!$this->aKataoMemberRelatedByMemberId->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoMemberRelatedByMemberId->getValidationFailures());
				}
			}

			if ($this->aKataoMemberRelatedByRefererId !== null) {
				if (!$this->aKataoMemberRelatedByRefererId->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoMemberRelatedByRefererId->getValidationFailures());
				}
			}


			if (($retval = KataoSupplierPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}


				if ($this->collKataoProducts !== null) {
					foreach($this->collKataoProducts as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoUsers !== null) {
					foreach($this->collKataoUsers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierNodes !== null) {
					foreach($this->collKataoSupplierNodes as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoNodeSuppliers !== null) {
					foreach($this->collKataoNodeSuppliers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoPeriodSuppliers !== null) {
					foreach($this->collKataoPeriodSuppliers as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoOrders !== null) {
					foreach($this->collKataoOrders as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierInvoices !== null) {
					foreach($this->collKataoSupplierInvoices as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}

				if ($this->collKataoSupplierProductCategorys !== null) {
					foreach($this->collKataoSupplierProductCategorys as $referrerFK) {
						if (!$referrerFK->validate($columns)) {
							$failureMap = array_merge($failureMap, $referrerFK->getValidationFailures());
						}
					}
				}


			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoSupplierPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoIndex();
				break;
			case 2:
				return $this->getMemberId();
				break;
			case 3:
				return $this->getRefererId();
				break;
			case 4:
				return $this->getName();
				break;
			case 5:
				return $this->getUrlIdentifier();
				break;
			case 6:
				return $this->getPresentation();
				break;
			case 7:
				return $this->getEmail();
				break;
			case 8:
				return $this->getAddress1();
				break;
			case 9:
				return $this->getAddress2();
				break;
			case 10:
				return $this->getZip();
				break;
			case 11:
				return $this->getCity();
				break;
			case 12:
				return $this->getPhone();
				break;
			case 13:
				return $this->getFax();
				break;
			case 14:
				return $this->getWebsite();
				break;
			case 15:
				return $this->getGmapLng();
				break;
			case 16:
				return $this->getGmapLat();
				break;
			case 17:
				return $this->getAccountingCode();
				break;
			case 18:
				return $this->getAccountingCodeSol();
				break;
			case 19:
				return $this->getSiretNumber();
				break;
			case 20:
				return $this->getIsGlobal();
				break;
			case 21:
				return $this->getOrderMinAmount();
				break;
			case 22:
				return $this->getCreatedAt();
				break;
			case 23:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoSupplierPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoIndex(),
			$keys[2] => $this->getMemberId(),
			$keys[3] => $this->getRefererId(),
			$keys[4] => $this->getName(),
			$keys[5] => $this->getUrlIdentifier(),
			$keys[6] => $this->getPresentation(),
			$keys[7] => $this->getEmail(),
			$keys[8] => $this->getAddress1(),
			$keys[9] => $this->getAddress2(),
			$keys[10] => $this->getZip(),
			$keys[11] => $this->getCity(),
			$keys[12] => $this->getPhone(),
			$keys[13] => $this->getFax(),
			$keys[14] => $this->getWebsite(),
			$keys[15] => $this->getGmapLng(),
			$keys[16] => $this->getGmapLat(),
			$keys[17] => $this->getAccountingCode(),
			$keys[18] => $this->getAccountingCodeSol(),
			$keys[19] => $this->getSiretNumber(),
			$keys[20] => $this->getIsGlobal(),
			$keys[21] => $this->getOrderMinAmount(),
			$keys[22] => $this->getCreatedAt(),
			$keys[23] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoSupplierPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoIndex($value);
				break;
			case 2:
				$this->setMemberId($value);
				break;
			case 3:
				$this->setRefererId($value);
				break;
			case 4:
				$this->setName($value);
				break;
			case 5:
				$this->setUrlIdentifier($value);
				break;
			case 6:
				$this->setPresentation($value);
				break;
			case 7:
				$this->setEmail($value);
				break;
			case 8:
				$this->setAddress1($value);
				break;
			case 9:
				$this->setAddress2($value);
				break;
			case 10:
				$this->setZip($value);
				break;
			case 11:
				$this->setCity($value);
				break;
			case 12:
				$this->setPhone($value);
				break;
			case 13:
				$this->setFax($value);
				break;
			case 14:
				$this->setWebsite($value);
				break;
			case 15:
				$this->setGmapLng($value);
				break;
			case 16:
				$this->setGmapLat($value);
				break;
			case 17:
				$this->setAccountingCode($value);
				break;
			case 18:
				$this->setAccountingCodeSol($value);
				break;
			case 19:
				$this->setSiretNumber($value);
				break;
			case 20:
				$this->setIsGlobal($value);
				break;
			case 21:
				$this->setOrderMinAmount($value);
				break;
			case 22:
				$this->setCreatedAt($value);
				break;
			case 23:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoSupplierPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoIndex($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setMemberId($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setRefererId($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setName($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setUrlIdentifier($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setPresentation($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setEmail($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setAddress1($arr[$keys[8]]);
		if (array_key_exists($keys[9], $arr)) $this->setAddress2($arr[$keys[9]]);
		if (array_key_exists($keys[10], $arr)) $this->setZip($arr[$keys[10]]);
		if (array_key_exists($keys[11], $arr)) $this->setCity($arr[$keys[11]]);
		if (array_key_exists($keys[12], $arr)) $this->setPhone($arr[$keys[12]]);
		if (array_key_exists($keys[13], $arr)) $this->setFax($arr[$keys[13]]);
		if (array_key_exists($keys[14], $arr)) $this->setWebsite($arr[$keys[14]]);
		if (array_key_exists($keys[15], $arr)) $this->setGmapLng($arr[$keys[15]]);
		if (array_key_exists($keys[16], $arr)) $this->setGmapLat($arr[$keys[16]]);
		if (array_key_exists($keys[17], $arr)) $this->setAccountingCode($arr[$keys[17]]);
		if (array_key_exists($keys[18], $arr)) $this->setAccountingCodeSol($arr[$keys[18]]);
		if (array_key_exists($keys[19], $arr)) $this->setSiretNumber($arr[$keys[19]]);
		if (array_key_exists($keys[20], $arr)) $this->setIsGlobal($arr[$keys[20]]);
		if (array_key_exists($keys[21], $arr)) $this->setOrderMinAmount($arr[$keys[21]]);
		if (array_key_exists($keys[22], $arr)) $this->setCreatedAt($arr[$keys[22]]);
		if (array_key_exists($keys[23], $arr)) $this->setUpdatedAt($arr[$keys[23]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoSupplierPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoSupplierPeer::ID)) $criteria->add(KataoSupplierPeer::ID, $this->id);
		if ($this->isColumnModified(KataoSupplierPeer::KATAO_INDEX)) $criteria->add(KataoSupplierPeer::KATAO_INDEX, $this->katao_index);
		if ($this->isColumnModified(KataoSupplierPeer::MEMBER_ID)) $criteria->add(KataoSupplierPeer::MEMBER_ID, $this->member_id);
		if ($this->isColumnModified(KataoSupplierPeer::REFERER_ID)) $criteria->add(KataoSupplierPeer::REFERER_ID, $this->referer_id);
		if ($this->isColumnModified(KataoSupplierPeer::NAME)) $criteria->add(KataoSupplierPeer::NAME, $this->name);
		if ($this->isColumnModified(KataoSupplierPeer::URL_IDENTIFIER)) $criteria->add(KataoSupplierPeer::URL_IDENTIFIER, $this->url_identifier);
		if ($this->isColumnModified(KataoSupplierPeer::PRESENTATION)) $criteria->add(KataoSupplierPeer::PRESENTATION, $this->presentation);
		if ($this->isColumnModified(KataoSupplierPeer::EMAIL)) $criteria->add(KataoSupplierPeer::EMAIL, $this->email);
		if ($this->isColumnModified(KataoSupplierPeer::ADDRESS1)) $criteria->add(KataoSupplierPeer::ADDRESS1, $this->address1);
		if ($this->isColumnModified(KataoSupplierPeer::ADDRESS2)) $criteria->add(KataoSupplierPeer::ADDRESS2, $this->address2);
		if ($this->isColumnModified(KataoSupplierPeer::ZIP)) $criteria->add(KataoSupplierPeer::ZIP, $this->zip);
		if ($this->isColumnModified(KataoSupplierPeer::CITY)) $criteria->add(KataoSupplierPeer::CITY, $this->city);
		if ($this->isColumnModified(KataoSupplierPeer::PHONE)) $criteria->add(KataoSupplierPeer::PHONE, $this->phone);
		if ($this->isColumnModified(KataoSupplierPeer::FAX)) $criteria->add(KataoSupplierPeer::FAX, $this->fax);
		if ($this->isColumnModified(KataoSupplierPeer::WEBSITE)) $criteria->add(KataoSupplierPeer::WEBSITE, $this->website);
		if ($this->isColumnModified(KataoSupplierPeer::GMAP_LNG)) $criteria->add(KataoSupplierPeer::GMAP_LNG, $this->gmap_lng);
		if ($this->isColumnModified(KataoSupplierPeer::GMAP_LAT)) $criteria->add(KataoSupplierPeer::GMAP_LAT, $this->gmap_lat);
		if ($this->isColumnModified(KataoSupplierPeer::ACCOUNTING_CODE)) $criteria->add(KataoSupplierPeer::ACCOUNTING_CODE, $this->accounting_code);
		if ($this->isColumnModified(KataoSupplierPeer::ACCOUNTING_CODE_SOL)) $criteria->add(KataoSupplierPeer::ACCOUNTING_CODE_SOL, $this->accounting_code_sol);
		if ($this->isColumnModified(KataoSupplierPeer::SIRET_NUMBER)) $criteria->add(KataoSupplierPeer::SIRET_NUMBER, $this->siret_number);
		if ($this->isColumnModified(KataoSupplierPeer::IS_GLOBAL)) $criteria->add(KataoSupplierPeer::IS_GLOBAL, $this->is_global);
		if ($this->isColumnModified(KataoSupplierPeer::ORDER_MIN_AMOUNT)) $criteria->add(KataoSupplierPeer::ORDER_MIN_AMOUNT, $this->order_min_amount);
		if ($this->isColumnModified(KataoSupplierPeer::CREATED_AT)) $criteria->add(KataoSupplierPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoSupplierPeer::UPDATED_AT)) $criteria->add(KataoSupplierPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoSupplierPeer::DATABASE_NAME);

		$criteria->add(KataoSupplierPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoSupplier (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoIndex($this->katao_index);

		$copyObj->setMemberId($this->member_id);

		$copyObj->setRefererId($this->referer_id);

		$copyObj->setName($this->name);

		$copyObj->setUrlIdentifier($this->url_identifier);

		$copyObj->setPresentation($this->presentation);

		$copyObj->setEmail($this->email);

		$copyObj->setAddress1($this->address1);

		$copyObj->setAddress2($this->address2);

		$copyObj->setZip($this->zip);

		$copyObj->setCity($this->city);

		$copyObj->setPhone($this->phone);

		$copyObj->setFax($this->fax);

		$copyObj->setWebsite($this->website);

		$copyObj->setGmapLng($this->gmap_lng);

		$copyObj->setGmapLat($this->gmap_lat);

		$copyObj->setAccountingCode($this->accounting_code);

		$copyObj->setAccountingCodeSol($this->accounting_code_sol);

		$copyObj->setSiretNumber($this->siret_number);

		$copyObj->setIsGlobal($this->is_global);

		$copyObj->setOrderMinAmount($this->order_min_amount);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		if ($deepCopy) {
			// important: temporarily setNew(false) because this affects the behavior of
			// the getter/setter methods for fkey referrer objects.
			$copyObj->setNew(false);

			foreach($this->getKataoProducts() as $relObj) {
				$copyObj->addKataoProduct($relObj->copy($deepCopy));
			}

			foreach($this->getKataoUsers() as $relObj) {
				$copyObj->addKataoUser($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierNodes() as $relObj) {
				$copyObj->addKataoSupplierNode($relObj->copy($deepCopy));
			}

			foreach($this->getKataoNodeSuppliers() as $relObj) {
				$copyObj->addKataoNodeSupplier($relObj->copy($deepCopy));
			}

			foreach($this->getKataoPeriodSuppliers() as $relObj) {
				$copyObj->addKataoPeriodSupplier($relObj->copy($deepCopy));
			}

			foreach($this->getKataoOrders() as $relObj) {
				$copyObj->addKataoOrder($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierInvoices() as $relObj) {
				$copyObj->addKataoSupplierInvoice($relObj->copy($deepCopy));
			}

			foreach($this->getKataoSupplierProductCategorys() as $relObj) {
				$copyObj->addKataoSupplierProductCategory($relObj->copy($deepCopy));
			}

		} // if ($deepCopy)


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoSupplier Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoSupplierPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoSupplierPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoMember object.
	 *
	 * @param      KataoMember $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoMemberRelatedByMemberId($v)
	{


		if ($v === null) {
			$this->setMemberId(NULL);
		} else {
			$this->setMemberId($v->getId());
		}


		$this->aKataoMemberRelatedByMemberId = $v;
	}


	/**
	 * Get the associated KataoMember object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoMember The associated KataoMember object.
	 * @throws     PropelException
	 */
	public function getKataoMemberRelatedByMemberId($con = null)
	{
		if ($this->aKataoMemberRelatedByMemberId === null && ($this->member_id !== null)) {
			// include the related Peer class
			$this->aKataoMemberRelatedByMemberId = KataoMemberPeer::retrieveByPK($this->member_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoMemberPeer::retrieveByPK($this->member_id, $con);
			   $obj->addKataoMembersRelatedByMemberId($this);
			 */
		}
		return $this->aKataoMemberRelatedByMemberId;
	}

	/**
	 * Declares an association between this object and a KataoMember object.
	 *
	 * @param      KataoMember $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoMemberRelatedByRefererId($v)
	{


		if ($v === null) {
			$this->setRefererId(NULL);
		} else {
			$this->setRefererId($v->getId());
		}


		$this->aKataoMemberRelatedByRefererId = $v;
	}


	/**
	 * Get the associated KataoMember object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoMember The associated KataoMember object.
	 * @throws     PropelException
	 */
	public function getKataoMemberRelatedByRefererId($con = null)
	{
		if ($this->aKataoMemberRelatedByRefererId === null && ($this->referer_id !== null)) {
			// include the related Peer class
			$this->aKataoMemberRelatedByRefererId = KataoMemberPeer::retrieveByPK($this->referer_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoMemberPeer::retrieveByPK($this->referer_id, $con);
			   $obj->addKataoMembersRelatedByRefererId($this);
			 */
		}
		return $this->aKataoMemberRelatedByRefererId;
	}

	/**
	 * Temporary storage of collKataoProducts to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoProducts()
	{
		if ($this->collKataoProducts === null) {
			$this->collKataoProducts = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoProducts from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoProducts($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoProducts === null) {
			if ($this->isNew()) {
			   $this->collKataoProducts = array();
			} else {

				$criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoProductPeer::addSelectColumns($criteria);
				$this->collKataoProducts = KataoProductPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoProductPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoProductCriteria) || !$this->lastKataoProductCriteria->equals($criteria)) {
					$this->collKataoProducts = KataoProductPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoProductCriteria = $criteria;
		return $this->collKataoProducts;
	}

	/**
	 * Returns the number of related KataoProducts.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoProducts($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoProductPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoProduct object to this object
	 * through the KataoProduct foreign key attribute
	 *
	 * @param      KataoProduct $l KataoProduct
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoProduct(KataoProduct $l)
	{
		$this->collKataoProducts[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoProducts from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoProductsJoinKataoProductFamily($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoProducts === null) {
			if ($this->isNew()) {
				$this->collKataoProducts = array();
			} else {

				$criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoProducts = KataoProductPeer::doSelectJoinKataoProductFamily($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoProductPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoProductCriteria) || !$this->lastKataoProductCriteria->equals($criteria)) {
				$this->collKataoProducts = KataoProductPeer::doSelectJoinKataoProductFamily($criteria, $con);
			}
		}
		$this->lastKataoProductCriteria = $criteria;

		return $this->collKataoProducts;
	}

	/**
	 * Temporary storage of collKataoUsers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoUsers()
	{
		if ($this->collKataoUsers === null) {
			$this->collKataoUsers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoUsers from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoUsers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoUsers === null) {
			if ($this->isNew()) {
			   $this->collKataoUsers = array();
			} else {

				$criteria->add(KataoUserPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoUserPeer::addSelectColumns($criteria);
				$this->collKataoUsers = KataoUserPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoUserPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoUserPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoUserCriteria) || !$this->lastKataoUserCriteria->equals($criteria)) {
					$this->collKataoUsers = KataoUserPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoUserCriteria = $criteria;
		return $this->collKataoUsers;
	}

	/**
	 * Returns the number of related KataoUsers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoUsers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoUserPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoUserPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoUser object to this object
	 * through the KataoUser foreign key attribute
	 *
	 * @param      KataoUser $l KataoUser
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoUser(KataoUser $l)
	{
		$this->collKataoUsers[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoUsers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoUsersJoinKataoMember($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoUsers === null) {
			if ($this->isNew()) {
				$this->collKataoUsers = array();
			} else {

				$criteria->add(KataoUserPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoUsers = KataoUserPeer::doSelectJoinKataoMember($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoUserPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoUserCriteria) || !$this->lastKataoUserCriteria->equals($criteria)) {
				$this->collKataoUsers = KataoUserPeer::doSelectJoinKataoMember($criteria, $con);
			}
		}
		$this->lastKataoUserCriteria = $criteria;

		return $this->collKataoUsers;
	}

	/**
	 * Temporary storage of collKataoSupplierNodes to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierNodes()
	{
		if ($this->collKataoSupplierNodes === null) {
			$this->collKataoSupplierNodes = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierNodes from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierNodes($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierNodes === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierNodes = array();
			} else {

				$criteria->add(KataoSupplierNodePeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierNodePeer::addSelectColumns($criteria);
				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierNodePeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierNodePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierNodeCriteria) || !$this->lastKataoSupplierNodeCriteria->equals($criteria)) {
					$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierNodeCriteria = $criteria;
		return $this->collKataoSupplierNodes;
	}

	/**
	 * Returns the number of related KataoSupplierNodes.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierNodes($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierNodePeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoSupplierNodePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierNode object to this object
	 * through the KataoSupplierNode foreign key attribute
	 *
	 * @param      KataoSupplierNode $l KataoSupplierNode
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierNode(KataoSupplierNode $l)
	{
		$this->collKataoSupplierNodes[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierNodes from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoSupplierNodesJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierNodes === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierNodes = array();
			} else {

				$criteria->add(KataoSupplierNodePeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierNodePeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoSupplierNodeCriteria) || !$this->lastKataoSupplierNodeCriteria->equals($criteria)) {
				$this->collKataoSupplierNodes = KataoSupplierNodePeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoSupplierNodeCriteria = $criteria;

		return $this->collKataoSupplierNodes;
	}

	/**
	 * Temporary storage of collKataoNodeSuppliers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoNodeSuppliers()
	{
		if ($this->collKataoNodeSuppliers === null) {
			$this->collKataoNodeSuppliers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoNodeSuppliers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
			   $this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoNodeSupplierPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
					$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;
		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Returns the number of related KataoNodeSuppliers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoNodeSuppliers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoNodeSupplierPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoNodeSupplier object to this object
	 * through the KataoNodeSupplier foreign key attribute
	 *
	 * @param      KataoNodeSupplier $l KataoNodeSupplier
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoNodeSupplier(KataoNodeSupplier $l)
	{
		$this->collKataoNodeSuppliers[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoNodeSuppliersJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoNodeSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoNodeSuppliersJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoNodeSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoNodeSuppliers = array();
			} else {

				$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoNodeSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoNodeSupplierCriteria) || !$this->lastKataoNodeSupplierCriteria->equals($criteria)) {
				$this->collKataoNodeSuppliers = KataoNodeSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoNodeSupplierCriteria = $criteria;

		return $this->collKataoNodeSuppliers;
	}

	/**
	 * Temporary storage of collKataoPeriodSuppliers to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoPeriodSuppliers()
	{
		if ($this->collKataoPeriodSuppliers === null) {
			$this->collKataoPeriodSuppliers = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoPeriodSuppliers from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoPeriodSuppliers($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodSuppliers === null) {
			if ($this->isNew()) {
			   $this->collKataoPeriodSuppliers = array();
			} else {

				$criteria->add(KataoPeriodSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoPeriodSupplierPeer::addSelectColumns($criteria);
				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoPeriodSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoPeriodSupplierPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoPeriodSupplierCriteria) || !$this->lastKataoPeriodSupplierCriteria->equals($criteria)) {
					$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoPeriodSupplierCriteria = $criteria;
		return $this->collKataoPeriodSuppliers;
	}

	/**
	 * Returns the number of related KataoPeriodSuppliers.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoPeriodSuppliers($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoPeriodSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoPeriodSupplierPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoPeriodSupplier object to this object
	 * through the KataoPeriodSupplier foreign key attribute
	 *
	 * @param      KataoPeriodSupplier $l KataoPeriodSupplier
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoPeriodSupplier(KataoPeriodSupplier $l)
	{
		$this->collKataoPeriodSuppliers[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoPeriodSuppliers from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoPeriodSuppliersJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoPeriodSuppliers === null) {
			if ($this->isNew()) {
				$this->collKataoPeriodSuppliers = array();
			} else {

				$criteria->add(KataoPeriodSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoPeriodSupplierPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoPeriodSupplierCriteria) || !$this->lastKataoPeriodSupplierCriteria->equals($criteria)) {
				$this->collKataoPeriodSuppliers = KataoPeriodSupplierPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoPeriodSupplierCriteria = $criteria;

		return $this->collKataoPeriodSuppliers;
	}

	/**
	 * Temporary storage of collKataoOrders to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoOrders()
	{
		if ($this->collKataoOrders === null) {
			$this->collKataoOrders = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoOrders($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
			   $this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoOrderPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
					$this->collKataoOrders = KataoOrderPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoOrderCriteria = $criteria;
		return $this->collKataoOrders;
	}

	/**
	 * Returns the number of related KataoOrders.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoOrders($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoOrderPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoOrder object to this object
	 * through the KataoOrder foreign key attribute
	 *
	 * @param      KataoOrder $l KataoOrder
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoOrder(KataoOrder $l)
	{
		$this->collKataoOrders[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoOrdersJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoOrders from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoOrdersJoinKataoNode($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoOrders === null) {
			if ($this->isNew()) {
				$this->collKataoOrders = array();
			} else {

				$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoNode($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoOrderPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoOrderCriteria) || !$this->lastKataoOrderCriteria->equals($criteria)) {
				$this->collKataoOrders = KataoOrderPeer::doSelectJoinKataoNode($criteria, $con);
			}
		}
		$this->lastKataoOrderCriteria = $criteria;

		return $this->collKataoOrders;
	}

	/**
	 * Temporary storage of collKataoSupplierInvoices to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierInvoices()
	{
		if ($this->collKataoSupplierInvoices === null) {
			$this->collKataoSupplierInvoices = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierInvoices from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierInvoices($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoices === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierInvoices = array();
			} else {

				$criteria->add(KataoSupplierInvoicePeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierInvoicePeer::addSelectColumns($criteria);
				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierInvoicePeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierInvoicePeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierInvoiceCriteria) || !$this->lastKataoSupplierInvoiceCriteria->equals($criteria)) {
					$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierInvoiceCriteria = $criteria;
		return $this->collKataoSupplierInvoices;
	}

	/**
	 * Returns the number of related KataoSupplierInvoices.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierInvoices($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierInvoicePeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoSupplierInvoicePeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierInvoice object to this object
	 * through the KataoSupplierInvoice foreign key attribute
	 *
	 * @param      KataoSupplierInvoice $l KataoSupplierInvoice
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierInvoice(KataoSupplierInvoice $l)
	{
		$this->collKataoSupplierInvoices[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierInvoices from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoSupplierInvoicesJoinKataoPeriod($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierInvoices === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierInvoices = array();
			} else {

				$criteria->add(KataoSupplierInvoicePeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierInvoicePeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoSupplierInvoiceCriteria) || !$this->lastKataoSupplierInvoiceCriteria->equals($criteria)) {
				$this->collKataoSupplierInvoices = KataoSupplierInvoicePeer::doSelectJoinKataoPeriod($criteria, $con);
			}
		}
		$this->lastKataoSupplierInvoiceCriteria = $criteria;

		return $this->collKataoSupplierInvoices;
	}

	/**
	 * Temporary storage of collKataoSupplierProductCategorys to save a possible db hit in
	 * the event objects are add to the collection, but the
	 * complete collection is never requested.
	 * @return     void
	 */
	public function initKataoSupplierProductCategorys()
	{
		if ($this->collKataoSupplierProductCategorys === null) {
			$this->collKataoSupplierProductCategorys = array();
		}
	}

	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierProductCategorys from storage.
	 * If this KataoSupplier is new, it will return
	 * an empty collection or the current collection, the criteria
	 * is ignored on a new object.
	 *
	 * @param      Connection $con
	 * @param      Criteria $criteria
	 * @throws     PropelException
	 */
	public function getKataoSupplierProductCategorys($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierProductCategorys === null) {
			if ($this->isNew()) {
			   $this->collKataoSupplierProductCategorys = array();
			} else {

				$criteria->add(KataoSupplierProductCategoryPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierProductCategoryPeer::addSelectColumns($criteria);
				$this->collKataoSupplierProductCategorys = KataoSupplierProductCategoryPeer::doSelect($criteria, $con);
			}
		} else {
			// criteria has no effect for a new object
			if (!$this->isNew()) {
				// the following code is to determine if a new query is
				// called for.  If the criteria is the same as the last
				// one, just return the collection.


				$criteria->add(KataoSupplierProductCategoryPeer::KATAO_SUPPLIER_ID, $this->getId());

				KataoSupplierProductCategoryPeer::addSelectColumns($criteria);
				if (!isset($this->lastKataoSupplierProductCategoryCriteria) || !$this->lastKataoSupplierProductCategoryCriteria->equals($criteria)) {
					$this->collKataoSupplierProductCategorys = KataoSupplierProductCategoryPeer::doSelect($criteria, $con);
				}
			}
		}
		$this->lastKataoSupplierProductCategoryCriteria = $criteria;
		return $this->collKataoSupplierProductCategorys;
	}

	/**
	 * Returns the number of related KataoSupplierProductCategorys.
	 *
	 * @param      Criteria $criteria
	 * @param      boolean $distinct
	 * @param      Connection $con
	 * @throws     PropelException
	 */
	public function countKataoSupplierProductCategorys($criteria = null, $distinct = false, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		$criteria->add(KataoSupplierProductCategoryPeer::KATAO_SUPPLIER_ID, $this->getId());

		return KataoSupplierProductCategoryPeer::doCount($criteria, $distinct, $con);
	}

	/**
	 * Method called to associate a KataoSupplierProductCategory object to this object
	 * through the KataoSupplierProductCategory foreign key attribute
	 *
	 * @param      KataoSupplierProductCategory $l KataoSupplierProductCategory
	 * @return     void
	 * @throws     PropelException
	 */
	public function addKataoSupplierProductCategory(KataoSupplierProductCategory $l)
	{
		$this->collKataoSupplierProductCategorys[] = $l;
		$l->setKataoSupplier($this);
	}


	/**
	 * If this collection has already been initialized with
	 * an identical criteria, it returns the collection.
	 * Otherwise if this KataoSupplier is new, it will return
	 * an empty collection; or if this KataoSupplier has previously
	 * been saved, it will retrieve related KataoSupplierProductCategorys from storage.
	 *
	 * This method is protected by default in order to keep the public
	 * api reasonable.  You can provide public methods for those you
	 * actually need in KataoSupplier.
	 */
	public function getKataoSupplierProductCategorysJoinKataoProductCategory($criteria = null, $con = null)
	{
		// include the Peer class
		if ($criteria === null) {
			$criteria = new Criteria();
		}
		elseif ($criteria instanceof Criteria)
		{
			$criteria = clone $criteria;
		}

		if ($this->collKataoSupplierProductCategorys === null) {
			if ($this->isNew()) {
				$this->collKataoSupplierProductCategorys = array();
			} else {

				$criteria->add(KataoSupplierProductCategoryPeer::KATAO_SUPPLIER_ID, $this->getId());

				$this->collKataoSupplierProductCategorys = KataoSupplierProductCategoryPeer::doSelectJoinKataoProductCategory($criteria, $con);
			}
		} else {
			// the following code is to determine if a new query is
			// called for.  If the criteria is the same as the last
			// one, just return the collection.

			$criteria->add(KataoSupplierProductCategoryPeer::KATAO_SUPPLIER_ID, $this->getId());

			if (!isset($this->lastKataoSupplierProductCategoryCriteria) || !$this->lastKataoSupplierProductCategoryCriteria->equals($criteria)) {
				$this->collKataoSupplierProductCategorys = KataoSupplierProductCategoryPeer::doSelectJoinKataoProductCategory($criteria, $con);
			}
		}
		$this->lastKataoSupplierProductCategoryCriteria = $criteria;

		return $this->collKataoSupplierProductCategorys;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoSupplier:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoSupplier::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoSupplier
