<?php

/**
 * Base class that represents a row from the 'katao_supplier_invoice_product' table.
 *
 * 
 *
 * @package    lib.model.om
 */
abstract class BaseKataoSupplierInvoiceProduct extends BaseObject  implements Persistent {


	/**
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var        KataoSupplierInvoiceProductPeer
	 */
	protected static $peer;


	/**
	 * The value for the id field.
	 * @var        int
	 */
	protected $id;


	/**
	 * The value for the katao_supplier_invoice_id field.
	 * @var        int
	 */
	protected $katao_supplier_invoice_id;


	/**
	 * The value for the katao_product_id field.
	 * @var        int
	 */
	protected $katao_product_id;


	/**
	 * The value for the product_reference field.
	 * @var        string
	 */
	protected $product_reference;


	/**
	 * The value for the product_name field.
	 * @var        string
	 */
	protected $product_name;


	/**
	 * The value for the product_price_euro field.
	 * @var        double
	 */
	protected $product_price_euro;


	/**
	 * The value for the product_margin field.
	 * @var        double
	 */
	protected $product_margin;


	/**
	 * The value for the product_tva_rate field.
	 * @var        double
	 */
	protected $product_tva_rate = 0;


	/**
	 * The value for the quantity field.
	 * @var        int
	 */
	protected $quantity = 1;


	/**
	 * The value for the product_price_total field.
	 * @var        double
	 */
	protected $product_price_total = 0;


	/**
	 * The value for the created_at field.
	 * @var        int
	 */
	protected $created_at;


	/**
	 * The value for the updated_at field.
	 * @var        int
	 */
	protected $updated_at;

	/**
	 * @var        KataoSupplierInvoice
	 */
	protected $aKataoSupplierInvoice;

	/**
	 * @var        KataoProduct
	 */
	protected $aKataoProduct;

	/**
	 * Flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInSave = false;

	/**
	 * Flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var        boolean
	 */
	protected $alreadyInValidation = false;

	/**
	 * Get the [id] column value.
	 * 
	 * @return     int
	 */
	public function getId()
	{

		return $this->id;
	}

	/**
	 * Get the [katao_supplier_invoice_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoSupplierInvoiceId()
	{

		return $this->katao_supplier_invoice_id;
	}

	/**
	 * Get the [katao_product_id] column value.
	 * 
	 * @return     int
	 */
	public function getKataoProductId()
	{

		return $this->katao_product_id;
	}

	/**
	 * Get the [product_reference] column value.
	 * 
	 * @return     string
	 */
	public function getProductReference()
	{

		return $this->product_reference;
	}

	/**
	 * Get the [product_name] column value.
	 * 
	 * @return     string
	 */
	public function getProductName()
	{

		return $this->product_name;
	}

	/**
	 * Get the [product_price_euro] column value.
	 * 
	 * @return     double
	 */
	public function getProductPriceEuro()
	{

		return $this->product_price_euro;
	}

	/**
	 * Get the [product_margin] column value.
	 * 
	 * @return     double
	 */
	public function getProductMargin()
	{

		return $this->product_margin;
	}

	/**
	 * Get the [product_tva_rate] column value.
	 * 
	 * @return     double
	 */
	public function getProductTvaRate()
	{

		return $this->product_tva_rate;
	}

	/**
	 * Get the [quantity] column value.
	 * 
	 * @return     int
	 */
	public function getQuantity()
	{

		return $this->quantity;
	}

	/**
	 * Get the [product_price_total] column value.
	 * 
	 * @return     double
	 */
	public function getProductPriceTotal()
	{

		return $this->product_price_total;
	}

	/**
	 * Get the [optionally formatted] [created_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getCreatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->created_at === null || $this->created_at === '') {
			return null;
		} elseif (!is_int($this->created_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->created_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [created_at] as date/time value: " . var_export($this->created_at, true));
			}
		} else {
			$ts = $this->created_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Get the [optionally formatted] [updated_at] column value.
	 * 
	 * @param      string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return     mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws     PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getUpdatedAt($format = 'Y-m-d H:i:s')
	{

		if ($this->updated_at === null || $this->updated_at === '') {
			return null;
		} elseif (!is_int($this->updated_at)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->updated_at);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse value of [updated_at] as date/time value: " . var_export($this->updated_at, true));
			}
		} else {
			$ts = $this->updated_at;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}

	/**
	 * Set the value of [id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->id !== $v) {
			$this->id = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::ID;
		}

	} // setId()

	/**
	 * Set the value of [katao_supplier_invoice_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoSupplierInvoiceId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_supplier_invoice_id !== $v) {
			$this->katao_supplier_invoice_id = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::KATAO_SUPPLIER_INVOICE_ID;
		}

		if ($this->aKataoSupplierInvoice !== null && $this->aKataoSupplierInvoice->getId() !== $v) {
			$this->aKataoSupplierInvoice = null;
		}

	} // setKataoSupplierInvoiceId()

	/**
	 * Set the value of [katao_product_id] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setKataoProductId($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->katao_product_id !== $v) {
			$this->katao_product_id = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID;
		}

		if ($this->aKataoProduct !== null && $this->aKataoProduct->getId() !== $v) {
			$this->aKataoProduct = null;
		}

	} // setKataoProductId()

	/**
	 * Set the value of [product_reference] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setProductReference($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->product_reference !== $v) {
			$this->product_reference = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_REFERENCE;
		}

	} // setProductReference()

	/**
	 * Set the value of [product_name] column.
	 * 
	 * @param      string $v new value
	 * @return     void
	 */
	public function setProductName($v)
	{

		// Since the native PHP type for this column is string,
		// we will cast the input to a string (if it is not).
		if ($v !== null && !is_string($v)) {
			$v = (string) $v; 
		}

		if ($this->product_name !== $v) {
			$this->product_name = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_NAME;
		}

	} // setProductName()

	/**
	 * Set the value of [product_price_euro] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductPriceEuro($v)
	{

		if ($this->product_price_euro !== $v) {
			$this->product_price_euro = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_EURO;
		}

	} // setProductPriceEuro()

	/**
	 * Set the value of [product_margin] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductMargin($v)
	{

		if ($this->product_margin !== $v) {
			$this->product_margin = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_MARGIN;
		}

	} // setProductMargin()

	/**
	 * Set the value of [product_tva_rate] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductTvaRate($v)
	{

		if ($this->product_tva_rate !== $v || $v === 0) {
			$this->product_tva_rate = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_TVA_RATE;
		}

	} // setProductTvaRate()

	/**
	 * Set the value of [quantity] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setQuantity($v)
	{

		// Since the native PHP type for this column is integer,
		// we will cast the input value to an int (if it is not).
		if ($v !== null && !is_int($v) && is_numeric($v)) {
			$v = (int) $v;
		}

		if ($this->quantity !== $v || $v === 1) {
			$this->quantity = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::QUANTITY;
		}

	} // setQuantity()

	/**
	 * Set the value of [product_price_total] column.
	 * 
	 * @param      double $v new value
	 * @return     void
	 */
	public function setProductPriceTotal($v)
	{

		if ($this->product_price_total !== $v || $v === 0) {
			$this->product_price_total = $v;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_TOTAL;
		}

	} // setProductPriceTotal()

	/**
	 * Set the value of [created_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setCreatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [created_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->created_at !== $ts) {
			$this->created_at = $ts;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::CREATED_AT;
		}

	} // setCreatedAt()

	/**
	 * Set the value of [updated_at] column.
	 * 
	 * @param      int $v new value
	 * @return     void
	 */
	public function setUpdatedAt($v)
	{

		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1 || $ts === false) { // in PHP 5.1 return value changes to FALSE
				throw new PropelException("Unable to parse date/time value for [updated_at] from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->updated_at !== $ts) {
			$this->updated_at = $ts;
			$this->modifiedColumns[] = KataoSupplierInvoiceProductPeer::UPDATED_AT;
		}

	} // setUpdatedAt()

	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param      ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param      int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return     int next starting column
	 * @throws     PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {

			$this->id = $rs->getInt($startcol + 0);

			$this->katao_supplier_invoice_id = $rs->getInt($startcol + 1);

			$this->katao_product_id = $rs->getInt($startcol + 2);

			$this->product_reference = $rs->getString($startcol + 3);

			$this->product_name = $rs->getString($startcol + 4);

			$this->product_price_euro = $rs->getFloat($startcol + 5);

			$this->product_margin = $rs->getFloat($startcol + 6);

			$this->product_tva_rate = $rs->getFloat($startcol + 7);

			$this->quantity = $rs->getInt($startcol + 8);

			$this->product_price_total = $rs->getFloat($startcol + 9);

			$this->created_at = $rs->getTimestamp($startcol + 10, null);

			$this->updated_at = $rs->getTimestamp($startcol + 11, null);

			$this->resetModified();

			$this->setNew(false);

			// FIXME - using NUM_COLUMNS may be clearer.
			return $startcol + 12; // 12 = KataoSupplierInvoiceProductPeer::NUM_COLUMNS - KataoSupplierInvoiceProductPeer::NUM_LAZY_LOAD_COLUMNS).

		} catch (Exception $e) {
			throw new PropelException("Error populating KataoSupplierInvoiceProduct object", $e);
		}
	}

	/**
	 * Removes this object from datastore and sets delete attribute.
	 *
	 * @param      Connection $con
	 * @return     void
	 * @throws     PropelException
	 * @see        BaseObject::setDeleted()
	 * @see        BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoSupplierInvoiceProduct:delete:pre') as $callable)
    {
      $ret = call_user_func($callable, $this, $con);
      if ($ret)
      {
        return;
      }
    }


		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoSupplierInvoiceProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			KataoSupplierInvoiceProductPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	

    foreach (sfMixer::getCallables('BaseKataoSupplierInvoiceProduct:delete:post') as $callable)
    {
      call_user_func($callable, $this, $con);
    }

  }
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        doSave()
	 */
	public function save($con = null)
	{

    foreach (sfMixer::getCallables('BaseKataoSupplierInvoiceProduct:save:pre') as $callable)
    {
      $affectedRows = call_user_func($callable, $this, $con);
      if (is_int($affectedRows))
      {
        return $affectedRows;
      }
    }


    if ($this->isNew() && !$this->isColumnModified(KataoSupplierInvoiceProductPeer::CREATED_AT))
    {
      $this->setCreatedAt(time());
    }

    if ($this->isModified() && !$this->isColumnModified(KataoSupplierInvoiceProductPeer::UPDATED_AT))
    {
      $this->setUpdatedAt(time());
    }

		if ($this->isDeleted()) {
			throw new PropelException("You cannot save an object that has been deleted.");
		}

		if ($con === null) {
			$con = Propel::getConnection(KataoSupplierInvoiceProductPeer::DATABASE_NAME);
		}

		try {
			$con->begin();
			$affectedRows = $this->doSave($con);
			$con->commit();
    foreach (sfMixer::getCallables('BaseKataoSupplierInvoiceProduct:save:post') as $callable)
    {
      call_user_func($callable, $this, $con, $affectedRows);
    }

			return $affectedRows;
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}

	/**
	 * Stores the object in the database.
	 *
	 * If the object is new, it inserts it; otherwise an update is performed.
	 * All related objects are also updated in this method.
	 *
	 * @param      Connection $con
	 * @return     int The number of rows affected by this insert/update and any referring fk objects' save() operations.
	 * @throws     PropelException
	 * @see        save()
	 */
	protected function doSave($con)
	{
		$affectedRows = 0; // initialize var to track total num of affected rows
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;


			// We call the save method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoSupplierInvoice !== null) {
				if ($this->aKataoSupplierInvoice->isModified()) {
					$affectedRows += $this->aKataoSupplierInvoice->save($con);
				}
				$this->setKataoSupplierInvoice($this->aKataoSupplierInvoice);
			}

			if ($this->aKataoProduct !== null) {
				if ($this->aKataoProduct->isModified()) {
					$affectedRows += $this->aKataoProduct->save($con);
				}
				$this->setKataoProduct($this->aKataoProduct);
			}


			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = KataoSupplierInvoiceProductPeer::doInsert($this, $con);
					$affectedRows += 1; // we are assuming that there is only 1 row per doInsert() which
										 // should always be true here (even though technically
										 // BasePeer::doInsert() can insert multiple rows).

					$this->setId($pk);  //[IMV] update autoincrement primary key

					$this->setNew(false);
				} else {
					$affectedRows += KataoSupplierInvoiceProductPeer::doUpdate($this, $con);
				}
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			$this->alreadyInSave = false;
		}
		return $affectedRows;
	} // doSave()

	/**
	 * Array of ValidationFailed objects.
	 * @var        array ValidationFailed[]
	 */
	protected $validationFailures = array();

	/**
	 * Gets any ValidationFailed objects that resulted from last call to validate().
	 *
	 *
	 * @return     array ValidationFailed[]
	 * @see        validate()
	 */
	public function getValidationFailures()
	{
		return $this->validationFailures;
	}

	/**
	 * Validates the objects modified field values and all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param      mixed $columns Column name or an array of column names.
	 * @return     boolean Whether all columns pass validation.
	 * @see        doValidate()
	 * @see        getValidationFailures()
	 */
	public function validate($columns = null)
	{
		$res = $this->doValidate($columns);
		if ($res === true) {
			$this->validationFailures = array();
			return true;
		} else {
			$this->validationFailures = $res;
			return false;
		}
	}

	/**
	 * This function performs the validation work for complex object models.
	 *
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @param      array $columns Array of column names to validate.
	 * @return     mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate($columns = null)
	{
		if (!$this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;

			$failureMap = array();


			// We call the validate method on the following object(s) if they
			// were passed to this object by their coresponding set
			// method.  This object relates to these object(s) by a
			// foreign key reference.

			if ($this->aKataoSupplierInvoice !== null) {
				if (!$this->aKataoSupplierInvoice->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoSupplierInvoice->getValidationFailures());
				}
			}

			if ($this->aKataoProduct !== null) {
				if (!$this->aKataoProduct->validate($columns)) {
					$failureMap = array_merge($failureMap, $this->aKataoProduct->getValidationFailures());
				}
			}


			if (($retval = KataoSupplierInvoiceProductPeer::doValidate($this, $columns)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}



			$this->alreadyInValidation = false;
		}

		return (!empty($failureMap) ? $failureMap : true);
	}

	/**
	 * Retrieves a field from the object by name passed in as a string.
	 *
	 * @param      string $name name
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     mixed Value of field.
	 */
	public function getByName($name, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoSupplierInvoiceProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->getByPosition($pos);
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @return     mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
			case 0:
				return $this->getId();
				break;
			case 1:
				return $this->getKataoSupplierInvoiceId();
				break;
			case 2:
				return $this->getKataoProductId();
				break;
			case 3:
				return $this->getProductReference();
				break;
			case 4:
				return $this->getProductName();
				break;
			case 5:
				return $this->getProductPriceEuro();
				break;
			case 6:
				return $this->getProductMargin();
				break;
			case 7:
				return $this->getProductTvaRate();
				break;
			case 8:
				return $this->getQuantity();
				break;
			case 9:
				return $this->getProductPriceTotal();
				break;
			case 10:
				return $this->getCreatedAt();
				break;
			case 11:
				return $this->getUpdatedAt();
				break;
			default:
				return null;
				break;
		} // switch()
	}

	/**
	 * Exports the object as an array.
	 *
	 * You can specify the key type of the array by passing one of the class
	 * type constants.
	 *
	 * @param      string $keyType One of the class type constants TYPE_PHPNAME,
	 *                        TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     an associative array containing the field names (as keys) and field values
	 */
	public function toArray($keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoSupplierInvoiceProductPeer::getFieldNames($keyType);
		$result = array(
			$keys[0] => $this->getId(),
			$keys[1] => $this->getKataoSupplierInvoiceId(),
			$keys[2] => $this->getKataoProductId(),
			$keys[3] => $this->getProductReference(),
			$keys[4] => $this->getProductName(),
			$keys[5] => $this->getProductPriceEuro(),
			$keys[6] => $this->getProductMargin(),
			$keys[7] => $this->getProductTvaRate(),
			$keys[8] => $this->getQuantity(),
			$keys[9] => $this->getProductPriceTotal(),
			$keys[10] => $this->getCreatedAt(),
			$keys[11] => $this->getUpdatedAt(),
		);
		return $result;
	}

	/**
	 * Sets a field from the object by name passed in as a string.
	 *
	 * @param      string $name peer name
	 * @param      mixed $value field value
	 * @param      string $type The type of fieldname the $name is of:
	 *                     one of the class type constants TYPE_PHPNAME,
	 *                     TYPE_COLNAME, TYPE_FIELDNAME, TYPE_NUM
	 * @return     void
	 */
	public function setByName($name, $value, $type = BasePeer::TYPE_PHPNAME)
	{
		$pos = KataoSupplierInvoiceProductPeer::translateFieldName($name, $type, BasePeer::TYPE_NUM);
		return $this->setByPosition($pos, $value);
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param      int $pos position in xml schema
	 * @param      mixed $value field value
	 * @return     void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
			case 0:
				$this->setId($value);
				break;
			case 1:
				$this->setKataoSupplierInvoiceId($value);
				break;
			case 2:
				$this->setKataoProductId($value);
				break;
			case 3:
				$this->setProductReference($value);
				break;
			case 4:
				$this->setProductName($value);
				break;
			case 5:
				$this->setProductPriceEuro($value);
				break;
			case 6:
				$this->setProductMargin($value);
				break;
			case 7:
				$this->setProductTvaRate($value);
				break;
			case 8:
				$this->setQuantity($value);
				break;
			case 9:
				$this->setProductPriceTotal($value);
				break;
			case 10:
				$this->setCreatedAt($value);
				break;
			case 11:
				$this->setUpdatedAt($value);
				break;
		} // switch()
	}

	/**
	 * Populates the object using an array.
	 *
	 * This is particularly useful when populating an object from one of the
	 * request arrays (e.g. $_POST).  This method goes through the column
	 * names, checking to see whether a matching key exists in populated
	 * array. If so the setByName() method is called for that column.
	 *
	 * You can specify the key type of the array by additionally passing one
	 * of the class type constants TYPE_PHPNAME, TYPE_COLNAME, TYPE_FIELDNAME,
	 * TYPE_NUM. The default key type is the column's phpname (e.g. 'authorId')
	 *
	 * @param      array  $arr     An array to populate the object from.
	 * @param      string $keyType The type of keys the array uses.
	 * @return     void
	 */
	public function fromArray($arr, $keyType = BasePeer::TYPE_PHPNAME)
	{
		$keys = KataoSupplierInvoiceProductPeer::getFieldNames($keyType);

		if (array_key_exists($keys[0], $arr)) $this->setId($arr[$keys[0]]);
		if (array_key_exists($keys[1], $arr)) $this->setKataoSupplierInvoiceId($arr[$keys[1]]);
		if (array_key_exists($keys[2], $arr)) $this->setKataoProductId($arr[$keys[2]]);
		if (array_key_exists($keys[3], $arr)) $this->setProductReference($arr[$keys[3]]);
		if (array_key_exists($keys[4], $arr)) $this->setProductName($arr[$keys[4]]);
		if (array_key_exists($keys[5], $arr)) $this->setProductPriceEuro($arr[$keys[5]]);
		if (array_key_exists($keys[6], $arr)) $this->setProductMargin($arr[$keys[6]]);
		if (array_key_exists($keys[7], $arr)) $this->setProductTvaRate($arr[$keys[7]]);
		if (array_key_exists($keys[8], $arr)) $this->setQuantity($arr[$keys[8]]);
		if (array_key_exists($keys[9], $arr)) $this->setProductPriceTotal($arr[$keys[9]]);
		if (array_key_exists($keys[10], $arr)) $this->setCreatedAt($arr[$keys[10]]);
		if (array_key_exists($keys[11], $arr)) $this->setUpdatedAt($arr[$keys[11]]);
	}

	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return     Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{
		$criteria = new Criteria(KataoSupplierInvoiceProductPeer::DATABASE_NAME);

		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::ID)) $criteria->add(KataoSupplierInvoiceProductPeer::ID, $this->id);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::KATAO_SUPPLIER_INVOICE_ID)) $criteria->add(KataoSupplierInvoiceProductPeer::KATAO_SUPPLIER_INVOICE_ID, $this->katao_supplier_invoice_id);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID)) $criteria->add(KataoSupplierInvoiceProductPeer::KATAO_PRODUCT_ID, $this->katao_product_id);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_REFERENCE)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_REFERENCE, $this->product_reference);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_NAME)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_NAME, $this->product_name);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_EURO)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_EURO, $this->product_price_euro);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_MARGIN)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_MARGIN, $this->product_margin);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_TVA_RATE)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_TVA_RATE, $this->product_tva_rate);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::QUANTITY)) $criteria->add(KataoSupplierInvoiceProductPeer::QUANTITY, $this->quantity);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_TOTAL)) $criteria->add(KataoSupplierInvoiceProductPeer::PRODUCT_PRICE_TOTAL, $this->product_price_total);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::CREATED_AT)) $criteria->add(KataoSupplierInvoiceProductPeer::CREATED_AT, $this->created_at);
		if ($this->isColumnModified(KataoSupplierInvoiceProductPeer::UPDATED_AT)) $criteria->add(KataoSupplierInvoiceProductPeer::UPDATED_AT, $this->updated_at);

		return $criteria;
	}

	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 *
	 * @return     Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{
		$criteria = new Criteria(KataoSupplierInvoiceProductPeer::DATABASE_NAME);

		$criteria->add(KataoSupplierInvoiceProductPeer::ID, $this->id);

		return $criteria;
	}

	/**
	 * Returns the primary key for this object (row).
	 * @return     int
	 */
	public function getPrimaryKey()
	{
		return $this->getId();
	}

	/**
	 * Generic method to set the primary key (id column).
	 *
	 * @param      int $key Primary key.
	 * @return     void
	 */
	public function setPrimaryKey($key)
	{
		$this->setId($key);
	}

	/**
	 * Sets contents of passed object to values from current object.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      object $copyObj An object of KataoSupplierInvoiceProduct (or compatible) type.
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @throws     PropelException
	 */
	public function copyInto($copyObj, $deepCopy = false)
	{

		$copyObj->setKataoSupplierInvoiceId($this->katao_supplier_invoice_id);

		$copyObj->setKataoProductId($this->katao_product_id);

		$copyObj->setProductReference($this->product_reference);

		$copyObj->setProductName($this->product_name);

		$copyObj->setProductPriceEuro($this->product_price_euro);

		$copyObj->setProductMargin($this->product_margin);

		$copyObj->setProductTvaRate($this->product_tva_rate);

		$copyObj->setQuantity($this->quantity);

		$copyObj->setProductPriceTotal($this->product_price_total);

		$copyObj->setCreatedAt($this->created_at);

		$copyObj->setUpdatedAt($this->updated_at);


		$copyObj->setNew(true);

		$copyObj->setId(NULL); // this is a pkey column, so set to default value

	}

	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param      boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 * @return     KataoSupplierInvoiceProduct Clone of current object.
	 * @throws     PropelException
	 */
	public function copy($deepCopy = false)
	{
		// we use get_class(), because this might be a subclass
		$clazz = get_class($this);
		$copyObj = new $clazz();
		$this->copyInto($copyObj, $deepCopy);
		return $copyObj;
	}

	/**
	 * Returns a peer instance associated with this om.
	 *
	 * Since Peer classes are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 *
	 * @return     KataoSupplierInvoiceProductPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new KataoSupplierInvoiceProductPeer();
		}
		return self::$peer;
	}

	/**
	 * Declares an association between this object and a KataoSupplierInvoice object.
	 *
	 * @param      KataoSupplierInvoice $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoSupplierInvoice($v)
	{


		if ($v === null) {
			$this->setKataoSupplierInvoiceId(NULL);
		} else {
			$this->setKataoSupplierInvoiceId($v->getId());
		}


		$this->aKataoSupplierInvoice = $v;
	}


	/**
	 * Get the associated KataoSupplierInvoice object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoSupplierInvoice The associated KataoSupplierInvoice object.
	 * @throws     PropelException
	 */
	public function getKataoSupplierInvoice($con = null)
	{
		if ($this->aKataoSupplierInvoice === null && ($this->katao_supplier_invoice_id !== null)) {
			// include the related Peer class
			$this->aKataoSupplierInvoice = KataoSupplierInvoicePeer::retrieveByPK($this->katao_supplier_invoice_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoSupplierInvoicePeer::retrieveByPK($this->katao_supplier_invoice_id, $con);
			   $obj->addKataoSupplierInvoices($this);
			 */
		}
		return $this->aKataoSupplierInvoice;
	}

	/**
	 * Declares an association between this object and a KataoProduct object.
	 *
	 * @param      KataoProduct $v
	 * @return     void
	 * @throws     PropelException
	 */
	public function setKataoProduct($v)
	{


		if ($v === null) {
			$this->setKataoProductId(NULL);
		} else {
			$this->setKataoProductId($v->getId());
		}


		$this->aKataoProduct = $v;
	}


	/**
	 * Get the associated KataoProduct object
	 *
	 * @param      Connection Optional Connection object.
	 * @return     KataoProduct The associated KataoProduct object.
	 * @throws     PropelException
	 */
	public function getKataoProduct($con = null)
	{
		if ($this->aKataoProduct === null && ($this->katao_product_id !== null)) {
			// include the related Peer class
			$this->aKataoProduct = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);

			/* The following can be used instead of the line above to
			   guarantee the related object contains a reference
			   to this object, but this level of coupling
			   may be undesirable in many circumstances.
			   As it can lead to a db query with many results that may
			   never be used.
			   $obj = KataoProductPeer::retrieveByPK($this->katao_product_id, $con);
			   $obj->addKataoProducts($this);
			 */
		}
		return $this->aKataoProduct;
	}


  public function __call($method, $arguments)
  {
    if (!$callable = sfMixer::getCallable('BaseKataoSupplierInvoiceProduct:'.$method))
    {
      throw new sfException(sprintf('Call to undefined method BaseKataoSupplierInvoiceProduct::%s', $method));
    }

    array_unshift($arguments, $this);

    return call_user_func_array($callable, $arguments);
  }


} // BaseKataoSupplierInvoiceProduct
